package Agromodel::DB;

use strict;
use DBI;
use vars qw ($dbh);

sub init {
	$dbh = DBI->connect($ENV{'AGROMODEL_DBURL'}, $ENV{'AGROMODEL_USER'}, $ENV{'AGROMODEL_PASSWORD'}, { 'AutoCommit' => 1, 'RaiseError' => 0 } );	
}

sub cleanup {
	if (defined $dbh) {
		$dbh->disconnect();
	}
}

sub prepare($$) {
	my ($pkg, $query) = @_;
	if (defined(my $sth = $dbh->prepare($query))) {
		return $sth;
	} else {
		return undef;
	}
}

sub ors_h($;@) {
	my $query = shift;
	if (defined(my $sth = _select($query, @_))) {
		my $r = $sth->fetchrow_hashref;
		$sth->finish;
		return $r;
	} else {	
		return undef;
	}
}

sub ors_a($;@) {
	my $query = shift;
	if (defined(my $sth = _select($query, @_))) {
		my $r = $sth->fetchrow_arrayref;
		$sth->finish;
		return $r;
	} else {	
		return undef;
	}
}

sub _select($;@) {
	my $query = shift;
	my $sth;
	if (not defined($sth = $dbh->prepare($query))) {
		warn "_select: prepare returned undef: \"$query\" (" . join(',', @_) . ")\n";
		return undef;
	} elsif (not $sth->execute(@_)) {
		warn "_select: execute returned false: \"$query\"\n";
		return undef;
	} else {
		return $sth;
	}
}
			
sub ors_s($;@) {
	my $query = shift;
	if (defined(my $sth = _select($query, @_))) {
		my $r;
		if (defined(my $row = $sth->fetchrow_arrayref)) {
			$r = $row->[0];
		}
		$sth->finish;
		return $r;
	} else {
		return undef;
	}
}

sub s_s ($;@) {
	my ($query, @args) =@_;
	ors_s($query,@args);
}

sub s_a($;@) {
	my $query = shift;
	if (defined(my $sth = _select($query, @_))) {
		my $rows = [];
		while(defined(my $row = $sth->fetchrow_arrayref)) {
			push(@$rows, $row->[0]);
		}
		$sth->finish;	
		return $rows;
	} else {
		return undef;
	}
}

sub s_hoh($$;@) {
	my $query = shift;
	my $key = shift;
	if (defined(my $sth = _select($query, @_))) {
		my $rows = {};
		while(defined(my $row = $sth->fetchrow_hashref)) {
			$rows->{$row->{$key}} = $row;
		}
		$sth->finish;
		return $rows;
	} else {
		return undef;
	}
}	

sub s_hash_array($$;@) {
	my $query = shift;
	my $key = shift;
	if (defined(my $sth = _select($query, @_))) {
		my $rows = {};
		while (defined(my $row = $sth->fetchrow_hashref)) {
			my $key_value = normalize_string($row->{$key});
			if (not defined $rows->{$key_value}) {
				$rows->{$key_value} = [];
			}

			push(@{$rows->{$key_value}}, $row);
		}
		$sth->finish;	
		return $rows;
	} else {
		return undef;
	}
}

sub s_aoa($;@) {
	my $query = shift;
	if (defined(my $sth = _select($query, @_))) {
		my $result = [];
		while(defined(my $row1 = $sth->fetchrow_arrayref)) {
			push(@$result, [ @$row1 ]);
		}
		$sth->finish;	
		return $result;
	} else {
		return undef;
	}
}

sub s_aoh($;@) {
	my $query = shift;
	if (defined(my $sth = _select($query, @_))) {
		my $rows = [];
		while(defined(my $row = $sth->fetchrow_hashref)) {
			push(@$rows, $row);
		}
		$sth->finish;	
		return $rows;
	} else {
		return undef;
	}
}

sub dds($;@) {
	my $query = shift;
	my $sth;
	if (not defined($sth = $dbh->prepare($query))) {
		warn "dds: prepare returned undef: \"$query\"\n";
		return undef;
	} else {
		my $r = $sth->execute(@_);
		$sth->finish;
		warn "dds: execute returned false: \"$query\".  Params: " . join(',', @_) . "\n" unless($r);
		return $r;
	}
}	

sub sqlesc($) {
	my $t = shift;
	return $dbh->quote($t);
}

sub normalize_string($) {
	my $string = shift;
        $string = lc($string);
        $string =~ s//a/g;
        $string =~ s//e/g;
        $string =~ s//i/g;
        $string =~ s//o/g;
        $string =~ s//u/g;
        $string =~ s//n/g;

	# Le hago trim
	$string =~ s/^\s+//;
	$string =~ s/\s+$//;

        return $string;
}

1;

sub BEGIN {
	init();
}

sub END {
	cleanup();
}
