package Agromodel::Loaders::Purge;

use base qw(Agromodel::Loaders::Base);
use strict;
use Archive::Zip;
use Cwd;

sub new($) {
	my $pkg  = shift;
	my $func = $pkg . "::new";

	my $tables = {
		'activitymgmt_scenario'                 =>      '',
		'croppingcycle'                         =>      '',
		'croppingcycle_contexts'                =>      '',
		'croppingcycle_activity'                =>      '',
		'croppingcycle_historicaladjustment'    =>      '',
		'croppingcycle_inputprices'             =>      '',
		'croppingcycle_region'                  =>      '',
		'croppingcycle_soiltype'                =>      '',
		'farm'                                  =>      '',
		'farmer'                                =>      '',
		'plot'                                  =>      '',
		'scenario'                              =>      '',
		'scenario_ensemble'                     =>      '',
		'yieldlookup'                           =>      ''
	};
	my $config = $pkg->getConfig();
	if (not defined $config) {
		warn "$func: Undefined configuration object";
		return undef();
	}

	my $data = {
		config	=>	$config,
		tables	=>	$tables,
		init_handlers	=>	[
			'purge'
		]
	};

	return bless($data, $pkg);
}

sub purge($) {
	my $self = shift;
	my $func = ref($self) . "::purge";
	$self->logInfo("Purging scenarios");

	my $config = $self->{config};
	my $mySqlBin = $config->{'fiuba.lamm.agromodel.db.mysql.bin'};
	my $mysqlUser = $config->{'fiuba.lamm.agromodel.db.user'};
	my $mysqlPassword = $config->{'fiuba.lamm.agromodel.db.password'};
	my $mysqlUrl = $config->{'fiuba.lamm.agromodel.db.url'};
	my ($mysqlHost, $mysqlDB);
	if ($mysqlUrl =~ /^jdbc:mysql:\/\/(.+?)(:\d+){0,1}\/(.+?)$/) {
		$mysqlHost = $1;
		$mysqlDB = $3;
	}
	$ENV{AGROMODEL_DBURL}     = "dbi:mysql:database=" . $mysqlDB . ";host=" . $mysqlHost . ";port=3306";
        $ENV{AGROMODEL_USER}      = $mysqlUser;
        $ENV{AGROMODEL_PASSWORD}  = $mysqlPassword;
	require Agromodel::DB;

	my $errors   = [];
	foreach my $tableName (keys %{$self->{tables}}) {
		my $deleteQuery = "delete from $mysqlDB.$tableName";

		$self->logInfo("Deleting scenarios from table $tableName");
		if (! Agromodel::DB::dds($deleteQuery)) {
			push(@$errors,  "Error executing query $deleteQuery");
		}
	}

	$self->logInfo("Optimizing tables");
	my $optimizeQuery = "optimize table " . join(", ", map { "$mysqlDB.$_" } keys(%{$self->{tables}}));
	if (! Agromodel::DB::dds($optimizeQuery)) {
		push(@$errors,  "Error optimizing tables");
	}

	$self->{errors}   = $errors;
	$self->{warnings} = [];

	return 1;
}

1;
