package fiuba.lamm.agromodel;

import java.util.*;

public class Activity {
	public static final Byte MAIZE = new Byte((byte) 1);
	public static final Byte SOYBEAN = new Byte((byte) 2);
	public static final Byte SOYWHEAT = new Byte((byte) 3);
		
	public enum Type { 
		Agriculture, 
		Cattle;
	
		public boolean isAgriculture() {
			return this.equals(Type.Agriculture);
		}
		
		public boolean isCattle() {
			return this.equals(Type.Cattle);
		}
	};
	
	protected Byte activityId;
	protected String activityName;
	protected Activity.Type activityType;
	protected String activityUnit;
	protected ActivityGroup activityGroup;
	protected Double harvestCostRate;
	protected Double marketingCostRate;
	protected Double aggregateCost;
	
	// Cropping cycle dependent attributes
	protected Map<Short,Double> outputPrice;
	
	public Activity(Byte activityId, String activityName, String activityType, String activityUnit, ActivityGroup activityGroup, Double harvestCostRate, Double marketingCostRate, Double aggregateCost) {
		this.activityId = activityId;
		this.activityName = activityName;
		this.activityType = Activity.Type.valueOf(activityType);
		this.activityUnit = activityUnit;
		this.activityGroup = activityGroup;
		this.harvestCostRate = harvestCostRate;
		this.marketingCostRate = marketingCostRate;
		this.aggregateCost = aggregateCost;
		this.outputPrice = new HashMap<Short,Double>();
	}
	
	public Byte getActivityId() {
		return this.activityId;
	}
	public String getActivityName() {
		return this.activityName;
	}
	public Activity.Type getActivityType() {
		return this.activityType;
	}
	public String getActivityUnit() {
		return this.activityUnit;
	}
	public ActivityGroup getActivityGroup() {
		return this.activityGroup;
	}
	public Double getHarvestCostRate() {
		return this.harvestCostRate;
	}
	public Double getMarketingCostRate() {
		return this.marketingCostRate;
	}
	public Double getAggregateCost() {
		return this.aggregateCost;
	}
	
	public Double getOutputPrice(Short croppingCycleId) {
		return this.outputPrice.get(croppingCycleId);
	}
	public void setOutputPrice(Short croppingCycleId, Double outputPrice) {
		this.outputPrice.put(croppingCycleId, outputPrice);
	}
	
	public int hashCode() {
		return this.activityId.hashCode();
	}
	public String toString() {
		return this.activityName;
	}
	public boolean equals(Object o) {
		if (o instanceof Activity) {
			Activity another = (Activity) o;
			return this.activityId.equals(another.activityId);
		}
		
		return false;
	}
}