package fiuba.lamm.agromodel;

import java.util.*;
import fiuba.lamm.agromodel.factories.*;
import fiuba.lamm.agromodel.finance.*;

public class ActivityManagement implements Comparable<ActivityManagement> {
	protected Short activityMgmtId;
	protected Activity activity;
	protected Region region;
	protected String activityMgmtName;
	protected String plantingDate;
	protected String genotype;
	protected Integer nFertilizerDose;
	protected Double rowSpacing;
	protected Double plantingDensity;
	protected ActivityManagement nextActivityManagement;
	protected ActivityManagement competingManagement;
	protected Map<InputType, Double> neededInputs;
	protected Map<LaborType, Integer> neededLabors;
	protected Map<DirectCost, Double> croppingCycleCosts;
	protected Map<Short,Map<Byte, Double>> croppingCycleYield;
	protected Short firstAvailableCroppingCycleId;
	protected boolean isAvailable;
	protected boolean isInnovation;
	protected Map<Short, Double> yieldAdjustmentIndex;
	protected Double directCostAdjustmentIndex;
		
	public ActivityManagement(Short activityMgmtId, Activity activity, Region region, String activityMgmtName, String plantingDate, String genotype, Integer nFertilizerDose, Double rowSpacing, Double plantingDensity, Map<InputType, Double> neededInputs, Map<LaborType, Integer> neededLabors, Short firstAvailableCroppingCycleId) {
		this.activityMgmtId = activityMgmtId;
		this.activity = activity;
		this.region = region;
		this.activityMgmtName = activityMgmtName;
		this.plantingDate = plantingDate;
		this.genotype = genotype;
		this.nFertilizerDose = nFertilizerDose;
		this.rowSpacing = rowSpacing;
		this.plantingDensity = plantingDensity;
		this.neededInputs = neededInputs;
		this.neededLabors = neededLabors;
		this.croppingCycleCosts = new HashMap<DirectCost, Double>();
		this.croppingCycleYield = new HashMap<Short, Map<Byte, Double>>();
		this.firstAvailableCroppingCycleId = firstAvailableCroppingCycleId;
		this.isAvailable = ((firstAvailableCroppingCycleId != null) && (firstAvailableCroppingCycleId <= 1)) ? true : false;
		this.isInnovation = false;
		this.yieldAdjustmentIndex = new HashMap<Short, Double>();
		this.directCostAdjustmentIndex = 1.0;
	}
	
	public Short getActivityMgmtId() {
		return this.activityMgmtId;
	}
	public Activity getActivity() {
		return this.activity;
	}
	public Region getRegion() {
		return this.region;
	}
	public String getActivityMgmtName() {
		return this.activityMgmtName;
	}
	public String getPlantingDate() {
		return this.plantingDate;
	}
	public String getGenotype() {
		return this.genotype;
	}
	public Integer getNFertilizerDose() {
		return this.nFertilizerDose;
	}
	public Double getRowSpacing() {
		return this.rowSpacing;
	}
	public Double getPlantingDensity() {
		return this.plantingDensity;
	}
	public Map<InputType, Double> getNeededInputs() {
		return this.neededInputs;
	}
	public Map<LaborType, Integer> getNeededLabors() {
		return this.neededLabors;
	}
	
	public void setNextActivityManagement(ActivityManagement nextActivityManagement) {
		this.nextActivityManagement = nextActivityManagement;
	}
	public ActivityManagement getNextActivityManagement() {
		return this.nextActivityManagement;
	}
	
	public Double getDirectCost(DirectCost costType) {
		Double cost = this.croppingCycleCosts.get(costType);
		if (costType.equals(DirectCost.FixedCost)) {
			cost *= this.directCostAdjustmentIndex;
		}
		
		return cost;
	}
	public void setCroppingCycleCosts(Short croppingCycleId, Map<DirectCost, Double> croppingCycleCosts) {
		this.croppingCycleCosts = croppingCycleCosts;
	}
	public void setDirectCostAdjustmentIndex(Double directCostAdjustmentIndex) {
		this.directCostAdjustmentIndex = directCostAdjustmentIndex;
	}
	
	public Double getCroppingCycleYield(Short croppingCycleId, Short technologyCroppingCycleId, Byte soilTypeId) {		
		return (this.yieldAdjustmentIndex.get(technologyCroppingCycleId) * this.croppingCycleYield.get(croppingCycleId).get(soilTypeId));
	}
	public void setCroppingCycleYields(Short croppingCycleId, Map<Byte, Double> yields) {
		this.croppingCycleYield.put(croppingCycleId, yields);
	}
	public void setYieldAdjustmentIndex(Short croppingCycleId, Double yieldAdjustmentIndex) {
		this.yieldAdjustmentIndex.put(croppingCycleId, yieldAdjustmentIndex);
	}
	
	public Short getFirstAvailableCroppingCycleId() {
		return this.firstAvailableCroppingCycleId;
	}
	
	public ActivityManagement getCompetingManagement() {
		return this.competingManagement;
	}
	public void setCompetingManagement(ActivityManagement competingManagement) {
		this.competingManagement = competingManagement;
	}
	
	public boolean isAvailable() {
		return this.isAvailable;
	}
	public void setIsAvailable(boolean availability) {
		this.isAvailable = availability;
	}
	
	public boolean isInnovation() {
		return this.isInnovation;
	}
	public void setIsInnovation(boolean isInnovation) {
		this.isInnovation = isInnovation;
	}
	
	/**
	 * Returns the next activity that should be cropped according to the following rotation sequence:
	 * Soy-Wheat	->		Maize
	 * Soy			->		Soy-Wheat
	 * Maize		->		Soy
	 * 
	 * If activity management is compound, then the last e/m is considered
	 * 		
	 * @return		The next activity in the rotation sequence.
	 */
	public Activity getNextActivityInRotationSequence() {
		ActivityFactory activityFactory = ActivityFactory.getInstance();
		if (this.activity.getActivityId().equals(Activity.MAIZE)) {
			return activityFactory.findByPrimaryKey(Activity.SOYBEAN);
		} else if (this.activity.getActivityId().equals(Activity.SOYBEAN)) {
			return activityFactory.findByPrimaryKey(Activity.SOYWHEAT);
		} else if (this.activity.getActivityId().equals(Activity.SOYWHEAT)) {
			return activityFactory.findByPrimaryKey(Activity.MAIZE);
		}
		
		return null;
	}
	
	public boolean equals(Object o) {
		if (o instanceof ActivityManagement) {
			ActivityManagement e = (ActivityManagement) o;
			return this.activityMgmtId.equals(e.getActivityMgmtId());
		}
		
		return false;
	}
	
	public int compareTo(ActivityManagement am) {
		return this.activityMgmtId.compareTo(am.activityMgmtId);
	}
	
	public int hashCode() {
		return this.activityMgmtId.hashCode();
	}
	
	public String toString() {
		return this.activityMgmtName;
	}
}