package fiuba.lamm.agromodel;

import repast.simphony.context.*;
import repast.simphony.dataLoader.*;
import repast.simphony.engine.environment.*;
import repast.simphony.parameter.*;
import repast.simphony.space.graph.*;
import fiuba.lamm.agromodel.agents.*;
import fiuba.lamm.agromodel.agents.factories.*;
import fiuba.lamm.agromodel.execution.*;
import fiuba.lamm.agromodel.execution.factories.*;
import fiuba.lamm.agromodel.networks.*;
import java.util.*;

public class ContextBuilderImpl implements ContextBuilder<Agent> {
	public Context<Agent> build(Context<Agent> context) {
		/* Cargo la configuracion inicial */
		long startTime = java.lang.System.currentTimeMillis();
		fiuba.lamm.agromodel.utils.System.init();		
		
		/*
		 * Get simulation parameters
		 */
		Parameters parameters = RunEnvironment.getInstance().getParameters();
		Short scenarioId = new Short(((Integer) parameters.getValue("scenarioId")).shortValue());
		Integer randomSeed = (Integer) parameters.getValue("randomSeed");
		
		/* Ahora creo el Manager y los Farmers */
		long initDbStartTime = java.lang.System.currentTimeMillis();
		List<Farm> farms = null;
		List<Farmer> farmers = null;
		Manager manager = null;
		Scenario scenario = null;
		try {
			// Create scenario instance and add it to RunEnvironment
			scenario = ScenarioFactory.getInstance().findByPrimaryKey(scenarioId);
			scenario.setRandomSeed(randomSeed);
			// FIXME: Esto no es Thread-Safe
			RunState.getInstance().addToRegistry("scenario", scenario);
			
			// Find all farmers and add them to the context
			FarmerFactory farmerFactory = FarmerFactory.getInstance(scenario);
			farmers = farmerFactory.findAll();
			for (Farmer farmer: farmers) {
				context.add(farmer);
			}
			
			// Create manager and add it to the context
			manager = Manager.newInstance(scenario, farmers);
			context.add(manager);
			
			// Find all farms, initialize aspiration level (default) and add them to the context
			FarmFactory farmFactory = FarmFactory.getInstance(scenario);
			farms = farmFactory.findAll();
			for (Farm farm: farms) {
				farm.setDefaultAspirationLevel();
				context.add(farm);
			}
			
			// Scenario has been loaded
			long initDbEndTime = java.lang.System.currentTimeMillis();
			long initDbTime = (initDbEndTime - initDbStartTime) / 1000;
			java.lang.System.out.println("Agents initilization time: " + initDbTime);			
		} catch (Exception e) {
			e.printStackTrace(java.lang.System.err);
			throw new FulminantError("Error al inicializar el modelo: " + e);			
		}
		
		// Create neighbourhood
		FarmNetworkModel farmNetworkModel = scenario.getFarmNetworkModel();
		Network<Agent> neighbourhood = farmNetworkModel.createNetwork(context, farms);
		manager.setNeighbourhood(neighbourhood);
		
		// Assign initial advisors
				
		// Ya tengo los farmers relacionados con los farms y con los plots
		long endTime = java.lang.System.currentTimeMillis();
		long diff = (endTime - startTime) / 1000;
		java.lang.System.out.println("Total initialization time: " + diff);
		
		// Set context Id
		context.setId("Agromodel");
		
		java.lang.System.gc(); // Run Garbage Collector after initialization
 		return context;
	}
}