package fiuba.lamm.agromodel;

import fiuba.lamm.agromodel.agents.*;
import fiuba.lamm.agromodel.decision.*;
import fiuba.lamm.agromodel.factories.*;
import java.sql.*;
import java.util.*;

public class CroppingCycle {
	protected Short croppingCycleId;
	protected String croppingCycleName;
	protected Double managementCost;
	protected Double structuralCost;
	protected Double utaPrice;
	protected Boolean isFirstActive;
	protected Boolean isActive;
	protected Map<SoilType,Map<ContextType,ContextTypeStatus>> expectedContexts;
	protected Map<SoilType,Map<ContextType,ContextTypeStatus>> actualContexts;
	
	public CroppingCycle(Short croppingCycleId, String croppingCycleName, Double managementCost, Double structuralCost, Double utaPrice, Boolean isActive) throws SQLException {
		this.croppingCycleId = croppingCycleId;
		this.croppingCycleName = croppingCycleName;
		this.managementCost = managementCost;
		this.structuralCost = structuralCost;
		this.utaPrice = utaPrice;
		this.isFirstActive = false;
		this.isActive = isActive;
		this.expectedContexts = new HashMap<SoilType,Map<ContextType,ContextTypeStatus>>();
		this.actualContexts = new HashMap<SoilType,Map<ContextType,ContextTypeStatus>>();
	}
	
	/* Definicion de accessors */
	public Short getCroppingCycleId() { return this.croppingCycleId; }
	public String getCroppingCycleName() { return this.croppingCycleName; }
	public Double getManagementCost() { return this.managementCost; }
	public Double getStructuralCost() { return this.structuralCost; }
	public Double getUtaPrice() { return this.utaPrice; }
	public Boolean isActive() { return this.isActive; }
	
	public ContextTypeStatus getExpectedContextTypeStatus(SoilType soilType, ContextType contextType) {
		return this.expectedContexts.get(soilType).get(contextType);
	}
	public void setExpectedContextTypeStatus(SoilType soilType, ContextType contextType, ContextTypeStatus status) {
		Map<ContextType,ContextTypeStatus> soilTypeStatus = null;
		if (! this.expectedContexts.containsKey(soilType)) {
			soilTypeStatus = new HashMap<ContextType,ContextTypeStatus>();
		} else {
			soilTypeStatus = this.expectedContexts.get(soilType);
		}
		
		soilTypeStatus.put(contextType, status);
		this.expectedContexts.put(soilType, soilTypeStatus);
	}
	
	public ContextTypeStatus getActualContextTypeStatus(SoilType soilType, ContextType contextType) {
		return this.actualContexts.get(soilType).get(contextType);
	}
	public void setActualContextTypeStatus(SoilType soilType, ContextType contextType, ContextTypeStatus status) {
		Map<ContextType,ContextTypeStatus> soilTypeStatus = null;
		if (! this.actualContexts.containsKey(soilType)) {
			soilTypeStatus = new HashMap<ContextType,ContextTypeStatus>();
		} else {
			soilTypeStatus = this.actualContexts.get(soilType);
		}
		
		soilTypeStatus.put(contextType, status);
		this.actualContexts.put(soilType, soilTypeStatus);
	}
	
	/* Metodos */
	public CroppingCycle nextCycle() throws SQLException {
		Manager manager = Manager.getInstance();		
		return CroppingCycleFactory.getInstance(manager.getScenario()).findNext(this);		
	}
	
	public boolean isFirstCroppingCycle() {
		Manager manager = Manager.getInstance();
		return this.equals(manager.getFirstActiveCroppingCycle());
	}
	
	public boolean isFirstActive() {
		return this.isFirstActive;
	}
	public void setIsFirstActive(boolean isFirstActive) {
		this.isFirstActive = isFirstActive;
	}
	
	public boolean equals(Object o) {
		if (o instanceof CroppingCycle) {
			CroppingCycle c = (CroppingCycle) o;
			return this.croppingCycleId.equals(c.getCroppingCycleId());
		}
		
		return false;
	}
	
	public int hashCode() {
		return this.croppingCycleId.hashCode();
	}
	
	public String toString() {
		return this.croppingCycleName;
	}
}