package fiuba.lamm.agromodel;

public enum OperationalStatus {
	LandLord	( "R", "LO" ),
	OwnerOnly	( "O", "OO" ),
	TenantOnly	( "T", "TO" ),
	OwnerTenant	( "B", "OT" ),
	Inactive	( "N", "IN" ),
	OutOfBusiness	( "U", "OB" ),
	RetiredOwner	( "E", "RO" ),
	RetiredTenant	( "I", "RT" ),
	Broke		( "K", "BR" );
	
	private String status;
	private String shortDescription;
	private OperationalStatus(String status, String shortDescription) {
		this.status = status;
		this.shortDescription = shortDescription;
	}
	
	public static OperationalStatus findByStatus(String status) {
		OperationalStatus oStatus = null;
		
		int i = 0;
		OperationalStatus os[] = OperationalStatus.values();
		while ((i < os.length) && (oStatus == null)) {
			if (os[i].status.equals(status)) oStatus = os[i];
			i++;
		}
		
		return oStatus;
	}
	
	public boolean isAlive() {
		return ! (this.equals(Broke) || this.equals(OutOfBusiness) || this.equals(RetiredTenant));
	}
	
	public boolean isActive() {
		return (this.isAlive() && !this.equals(Inactive));
	}
	
	public boolean isOperating() {
		return (this.isActive() && ! (this.equals(LandLord) || this.equals(RetiredOwner)));
	}
	
	public boolean isLiveOff() {
		return (this.equals(LandLord) || this.equals(RetiredOwner));
	}
	
	public boolean isRetired() {
		return (this.equals(RetiredTenant) || this.equals(RetiredOwner));
	}
	
	public String toString() {
		return this.shortDescription;
	}	
}