package fiuba.lamm.agromodel;

import fiuba.lamm.agromodel.agents.*;
import fiuba.lamm.agromodel.output.*;

@ExportableClass(minimumAllowedVerbosity = Verbosity.Normal)
public class Plot {
	protected Farm farm;
	protected SoilType soilType;
	protected Double plotSize;
	protected Byte plotNumber;
	protected ActivityManagement currentActivityManagement;
	
	/* Constructor for creating "generic" plots to be used in optimization process */
	public Plot(SoilType soilType, Double plotSize, ActivityManagement currentActivityManagement) {
		this.farm = null;
		this.soilType = soilType;
		this.plotSize = plotSize;
		this.plotNumber = null;
		this.currentActivityManagement = currentActivityManagement;
	}
	
	/* Constructor for being used in PlotFactory */
	public Plot(Farm farm, Byte plotNumber, SoilType soilType, Double plotSize) {
		this.farm = farm;
		this.soilType = null;
		this.plotSize = plotSize;
		this.plotNumber = plotNumber;
		this.currentActivityManagement = null;
	}
	
	public Integer getFarmId() {
		return this.farm.getFarmId();
	}
	
	public Farm getFarm() {
		return this.farm;
	}
	
	public SoilType getSoilType() {
		return ((this.soilType != null) ? this.soilType : this.farm.getSoilType());
	}
	
	@ExportableField(title = "Plot size", order = 2, minimumAllowedVerbosity = Verbosity.Normal)
	public Double getPlotSize() {
		return this.plotSize;
	}
	public void setPlotSize(Double plotSize) {
		this.plotSize = plotSize;
	}
	
	public Byte getPlotNumber() {
		return this.plotNumber;
	}
		
	@ExportableField(title = "Plot name", order = 1, minimumAllowedVerbosity = Verbosity.Normal)
	public String getPlotName() {
		return (this.farm.getFarmId() + "." + this.plotNumber);
	}
	
	@ExportableField(title = "Plot use Id", order = 3, minimumAllowedVerbosity = Verbosity.Normal)
	public Short getCurrentActivityManagementId() {
		return this.currentActivityManagement.getActivityMgmtId();
	}
	
	public ActivityManagement getCurrentActivityManagement() {
		return this.currentActivityManagement;
	}
	public void setCurrentActivityManagement(ActivityManagement activityManagement) {
		this.currentActivityManagement = activityManagement;
	}
	
	public int hashCode() {
		return this.farm.hashCode();
	}
}