package fiuba.lamm.agromodel;

import fiuba.lamm.agromodel.agents.*;
import fiuba.lamm.agromodel.factories.*;
import fiuba.lamm.agromodel.finance.*;

public class Region implements Comparable<Region> {
	protected Byte regionId;
	protected String regionName;
	protected Double distanceToPort;
	protected Double totalNoEaps;
	
	// Cropping cycle dependent attributes
	protected Double transportationCost;
	
	public Region(Byte regionId, String regionName, Double distanceToPort, Double totalNoEaps) {
		this.regionId = regionId;
		this.regionName = regionName;
		this.distanceToPort = distanceToPort;
		this.totalNoEaps = totalNoEaps;
		this.transportationCost = null;
	}
	
	/* En este caso no hay metodos set porque no tiene sentido modificar atributos de una zona */
	public Byte getRegionId() {
		return this.regionId;
	}
	public String getRegionName() {
		return this.regionName;
	}
	public Double getDistanceToPort() {
		return this.distanceToPort;
	}
	public Double getTotalNoEaps() {
		return this.totalNoEaps;
	}
	
	public Double getTransportationCost() {
		return this.transportationCost;
	}
	public void setTransportationCost(Double transportationCost) {
		this.transportationCost = transportationCost;
	}
	
	/**
	 * Finds the most expensive AM for this region and the current cropping cycle and calculates
	 * the operating 1ha with that AM
	 * 
	 * @param croppingCycle	Current cropping cycle
	 * @return				Cost of operating 1ha in this region
	 */
	public Double getExpectedOperationCost(CroppingCycle croppingCycle) {
		Manager manager = Manager.getInstance();
		ActivityManagement mostExpensiveManagement = ActivityManagementFactory.getInstance(manager.getScenario()).findMostExpensive(croppingCycle, this);
		
		double fixedCost = 0.0;
		while (mostExpensiveManagement != null) {
			fixedCost += mostExpensiveManagement.getDirectCost(DirectCost.FixedCost);
			mostExpensiveManagement = mostExpensiveManagement.getNextActivityManagement();
		}
		
		return fixedCost;
	}
	
	public boolean equals(Object o) {
		if (o instanceof Region) {
			Region r = (Region) o;
			return this.regionId.equals(r.getRegionId());
		}
		
		return false;
	}
	
	public int hashCode() {
		return this.regionId.hashCode();
	}
	
	public int compareTo(Region region) {
		return this.regionId.compareTo(region.getRegionId());
	}
}