package fiuba.lamm.agromodel.agents.factories;

import java.sql.*;
import java.util.*;
import fiuba.lamm.agromodel.*;
import fiuba.lamm.agromodel.agents.*;
import fiuba.lamm.agromodel.execution.*;
import fiuba.lamm.agromodel.factories.*;

public class AdvisorFactory extends Factory {
	protected Map<Short, Advisor> cache;	
	protected static Map<Scenario,AdvisorFactory> factories;
	
	public static synchronized AdvisorFactory getInstance(Scenario scenario) throws SQLException {
		if (factories == null) {
			factories = new TreeMap<Scenario,AdvisorFactory>();			
		}
		if (! factories.containsKey(scenario)) {
			factories.put(scenario, new AdvisorFactory(scenario));
		}
		
		return factories.get(scenario);
	}
	
	protected AdvisorFactory(Scenario scenario) throws SQLException {
		super();
		this.cache = new TreeMap<Short, Advisor>();
		
		RegionFactory regionFactory = RegionFactory.getInstance();
		InnovatorTypeFactory innovatorTypeFactory = InnovatorTypeFactory.getInstance();
		
		Connection connection = getConnection();
		PreparedStatement st = connection.prepareStatement("select * from advisor where scenarioId = ?");
		st.setShort(1, scenario.getScenarioId());
		ResultSet rs = st.executeQuery();
		
		while (rs.next()) {			
			Advisor advisor = new Advisor(
					rs.getShort("advisorId"),
					regionFactory.findByPrimaryKey(rs.getByte("regionId")),
					rs.getString("advisorName"),
					rs.getDouble("advisorFee"),
					innovatorTypeFactory.findByPrimaryKey(rs.getByte("innovatorTypeId"))
			);
			
			this.cache.put(advisor.getAdvisorId(), advisor);
		}
		rs.close();
		st.close();
		connection.close();
	}
	
	public Advisor findByPrimaryKey(Short advisorId) {
		return this.cache.get(advisorId);
	}
	
	public static void clearScenarioCache(Scenario scenario) {
		factories.remove(scenario);
	}
	
	public List<Advisor> findAll() {
		return new ArrayList<Advisor>(this.cache.values());
	}
	
	public List<Advisor> findByRegion(Region region) {
		List<Advisor> advisors = new ArrayList<Advisor>();
		List<Advisor> all = new ArrayList<Advisor>(this.cache.values());
		
		for (Advisor advisor: all) {
			if (advisor.getRegion().equals(region)) {
				advisors.add(advisor);
			}
		}
		
		return advisors;
	}	
}