package fiuba.lamm.agromodel.areaupdate;

import java.util.*;
import fiuba.lamm.agromodel.agents.*;

public class FarmSubSetCollection implements Comparable<FarmSubSetCollection> {
	protected Farmer farmer;
	protected List<Farm> farms;
	protected Double expectedGrossMargin;
	protected Double expectedOperationCost;
	
	public FarmSubSetCollection(Farmer farmer, Collection<Farm> farms) {
		this.farmer = farmer;
		this.farms = new ArrayList<Farm>(farms);
		this.expectedGrossMargin = 0.0;
		this.expectedOperationCost = 0.0;
		
		Iterator<Farm> iterator = farms.iterator();
		while (iterator.hasNext()) {
			Farm farm = iterator.next();
			this.expectedGrossMargin += farm.getExpectedGrossMargin();
			this.expectedOperationCost += farm.getExpectedOperationCost(this.farmer);
		}
	}
	
	public FarmSubSetCollection(Farmer farmer) {
		this.farmer = farmer;
		this.farms = new ArrayList<Farm>();
		this.expectedGrossMargin = 0.0;
		this.expectedOperationCost = 0.0;		
	}
	
	public Farmer getFarmer() {
		return this.farmer;
	}
	
	public List<Farm> getFarms() {
		return this.farms;
	}
	public boolean addFarm(Farm farm) {
		// Adds farms if not already contained in the collection
		// Returns true if farm was added, otherwise, false
		if (! this.farms.contains(farm)) {
			this.farms.add(farm);
			this.expectedGrossMargin += farm.getExpectedGrossMargin();
			this.expectedOperationCost += farm.getExpectedOperationCost(this.farmer);
			return true;
		}
		
		return false;
	}
	public Double getExpectedGrossMargin() {
		return this.expectedGrossMargin;
	}
	public Double getExpectedOperationCost() {
		return this.expectedOperationCost;
	}
	public boolean contains(Farm farm) {
		return this.farms.contains(farm);
	}
	public boolean containsAll(List<Farm> farms) {
		boolean contains = true;
		int i = 0;
		while ((i < farms.size()) && contains) {
			contains = this.contains(farms.get(i));
			i++;
		}
		
		return contains;
	}
	public boolean equals(Object o) {
		if (o instanceof FarmSubSetCollection) {
			FarmSubSetCollection aCollection = (FarmSubSetCollection) o;
			if (this.farms.size() == aCollection.farms.size()) {
				boolean e = true;
				int i = 0;
				while ((i < this.farms.size()) && e) {
					Farm farm = this.farms.get(i);
					e = aCollection.contains(farm);
					i++;
				}
				
				return e;
			}
		}
		
		return false;
	}
	
	public String toString() {
		String s = new String();
		for (int i = 0; i < this.farms.size(); i++) {
			s += (i > 0) ? ", " : "";
			s += this.farms.get(i).getFarmId();
		}
		
		return s;
	}
	
	/*
	 * Farm collections are ordered by:
	 * i. Expected gross margin descending
	 * ii. Expected operation cost ascending
	 */
	public int compareTo(FarmSubSetCollection aCollection) {
		if (this.expectedGrossMargin != aCollection.expectedGrossMargin) {
			return (- this.expectedGrossMargin.compareTo(aCollection.expectedGrossMargin));
		} else {
			return this.expectedOperationCost.compareTo(aCollection.expectedOperationCost);
		}
	}	
}
