package fiuba.lamm.agromodel.areaupdate.mcp;

import fiuba.lamm.agromodel.agents.*;
import fiuba.lamm.agromodel.areaupdate.*;
import java.util.*;

public class MCPLandMarketModel extends LandMarketModel {
	public SupplyDemandCreationComponent supplyDemandCreationComponent;
	
	public MCPLandMarketModel() {
		super();
		
		// 1. Definition of the components' execution time
		//    5 stages: Definition of the model, willingness values formation, MCP calculation, supply/demand creation, supply/demand matching
		Double componentExecutionTime = (LandMarketModel.LandMarketModleRelativeEndTime - LandMarketModel.LandMarketModelRelativeStartTime) / 5.0;

		// 2. Willingness values formation
		Double componentExecutionStartTime = componentExecutionTime;
		this.components.add(new WillingnessValuesFormationComponent(componentExecutionStartTime, componentExecutionStartTime + componentExecutionTime));
			
		// 3. MCP Calculation
		componentExecutionStartTime += componentExecutionTime;
		this.components.add(new MCPCalculationComponent(componentExecutionStartTime, componentExecutionStartTime + componentExecutionTime));
		
		// 4. Supply/demand list creation component
		componentExecutionStartTime += componentExecutionTime;
		this.supplyDemandCreationComponent = new SupplyDemandCreationComponent(componentExecutionStartTime, componentExecutionStartTime + componentExecutionTime);
		this.components.add(this.supplyDemandCreationComponent);
		
		// 5. Supply/demand matching component
		componentExecutionStartTime += componentExecutionTime;
		this.components.add(new fiuba.lamm.agromodel.areaupdate.mcp.SupplyDemandMatchingComponent(this, componentExecutionStartTime, componentExecutionStartTime + componentExecutionTime));
	}
	
	public List<Farm> getSupplyList() {
		return this.supplyDemandCreationComponent.getSupplyList();
	}
	
	public List<Farmer> getActiveDemandList() {
		return this.supplyDemandCreationComponent.getActiveDemandList();
	}
	
	public List<Farmer> getInactiveDemandList() {
		return this.supplyDemandCreationComponent.getInactiveDemandList();
	}	
}