package fiuba.lamm.agromodel.areaupdate.mcp.pe;

import fiuba.lamm.agromodel.agents.*;
import fiuba.lamm.agromodel.execution.*;

public abstract class ProgressEvaluationMechanism {
	/** 
	 * @param		farmer: Farmer whose progress is being evaluated
	 * @return		true if progress is acceptable, false otherwise
	 */
	public abstract boolean evaluate(Farmer farmer);
	
	/**
	 * Busca el mecanismo de evualuacion de progreso adecuado. Si el escenario tiene el atributo
	 * ProgressRateNOutOfM, entonces devuelve una instancia de la clase NOutOfMEvaluationMechanism.
	 * De lo contrario, devuelve una instancia de ThresholdEvaluationMechanism
	 * 
	 * @return		Segun lo indicado anteriormente	
	 */
	public static ProgressEvaluationMechanism getInstance() {
		Manager manager = Manager.getInstance();
		Scenario scenario = manager.getScenario();
		Integer croppingCyclesForProgressEvaluation = (Integer) scenario.getParameter("ProgressEvaluation.CroppingCyclesForProgressEvaluation");
		if (manager.getScenario().hasAttribute(Scenario.Attribute.ProgressRateNOutOfM)) {
			Integer minimumPositiveCycles = (Integer) scenario.getParameter("ProgressEvaluation.MinimumPositiveCycles");
			return new NOutOfMEvaluationMechanism(croppingCyclesForProgressEvaluation, minimumPositiveCycles);
		} else {
			return new ThresholdEvaluationMechanism(croppingCyclesForProgressEvaluation);
		}
	}
}
