package fiuba.lamm.agromodel.areaupdate.mcp.pe;

import fiuba.lamm.agromodel.agents.*;
import fiuba.lamm.agromodel.math.custom.*;
import java.util.*;

public class ThresholdEvaluationMechanism extends ProgressEvaluationMechanism {
	protected Integer croppingCyclesForProgressEvaluation;
	
	public ThresholdEvaluationMechanism(Integer croppingCyclesForProgressEvaluation) {
		this.croppingCyclesForProgressEvaluation = croppingCyclesForProgressEvaluation;
	}
	
	public boolean evaluate(Farmer farmer) {
		Double minimumProgressRate = farmer.getOwnMinimumProgressRate();
		
		if (minimumProgressRate != null) {
			List<Double> totalNetFarmingIncomes = farmer.getTotalNetFarmingIncomes();
			if (totalNetFarmingIncomes.size() >= croppingCyclesForProgressEvaluation) {
				CapitalProgressRateFunction progressRateFunction = new CapitalProgressRateFunction();
				
				Double windowEndingTotalNetFarmingIncome = totalNetFarmingIncomes.get(totalNetFarmingIncomes.size() - 1);
				Double windowBegginingTotalNetFarmingIncome = totalNetFarmingIncomes.get(totalNetFarmingIncomes.size() - croppingCyclesForProgressEvaluation);
				Double progressRate = progressRateFunction.apply(new double[] { windowBegginingTotalNetFarmingIncome, windowEndingTotalNetFarmingIncome });
				return (progressRate >= minimumProgressRate);
			} else {
				// Hay menos ciclos de los necesarios. Se considera que el progreso es aceptable
				return true;
			}
		} else {
			// El farmer no tiene minimum progress rate definido, por lo tanto se considera que el progreso
			// siempre es aceptable.
			return true;
		}
	}
}
