package fiuba.lamm.agromodel.areaupdate.mcp.wta;

import fiuba.lamm.agromodel.*;
import fiuba.lamm.agromodel.agents.*;
import fiuba.lamm.agromodel.decision.*;
import fiuba.lamm.agromodel.decision.factories.*;
import fiuba.lamm.agromodel.execution.*;
import java.sql.*;
import java.util.*;

public class SimpleWTACalculationMechanism extends WTACalculationMechanism {
	// El WTA se calcula como el precio de alquiler del ciclo antearior modificado por los contextos esperados
	public Map<Farm,Double> calculateWTA(Farmer farmer) {
		Map<Farm,Double> wtas = new HashMap<Farm,Double>();
		List<Farm> farms = farmer.getOwnedFarms();
		for (Farm farm: farms) {
			SoilType soilType = farm.getSoilType();
			Manager manager = Manager.getInstance();
			CroppingCycle croppingCycle = manager.getCurrentCroppingCycle();
			
			Double factor = 0.0;
			try {
				List<ContextType> contextTypes = ContextTypeFactory.getInstance(manager.getScenario()).findAll();
				for (ContextType contextType: contextTypes) {
					ContextTypeStatus expectedContextStatus = croppingCycle.getExpectedContextTypeStatus(soilType, contextType);
					factor += contextType.getAdjustmentFactor(soilType, expectedContextStatus);
				}
			} catch (SQLException sqle) {
				throw new FulminantError("Error al buscar el factory de ContextType: " + sqle);
			}
				
			Double wta = soilType.getRentalPrice() * (1.0 + factor);
			wtas.put(farm, wta);
		}
		
		return wtas;
	}
}
