package fiuba.lamm.agromodel.areaupdate.mcp.wta;

import fiuba.lamm.agromodel.agents.*;
import fiuba.lamm.agromodel.execution.*;
import java.util.*;

public abstract class WTACalculationMechanism {
	public enum Type { 
		SimpleWTACalculationMechanism	(0), 
		OptimizedWTACalculationMechanism	(1);
	
		private int id;
		Type(int id) {
			this.id = id;
		}
		
		public static Type byId(int id) {
			Type type = null;
			int i = 0;
			Type types[] = Type.values();
			
			while ((i < types.length) && (type == null)) {
				if (types[i].id == id) type = types[i];
				i++;
			}
			
			return type;
		}
	};
	
	public static WTACalculationMechanism getInstance(int wtaCalculationMechanismId) {
		Type type = Type.byId(wtaCalculationMechanismId);
		WTACalculationMechanism mechanism = null;
		
		if (type.equals(Type.SimpleWTACalculationMechanism)) {
			mechanism = new SimpleWTACalculationMechanism();
		} else if (type.equals(Type.OptimizedWTACalculationMechanism)) {
			mechanism = new OptimizedWTACalculationMechanism();
		} else {
			throw new FulminantError("Unknown WTACalculationMechanism: " + wtaCalculationMechanismId);
		}
		
		return mechanism;
	}
		
	public abstract Map<Farm,Double> calculateWTA(Farmer farmer);
}
