package fiuba.lamm.agromodel.decision;

import fiuba.lamm.agromodel.*;
import java.util.*;

public class ContextType {
	protected Short contextTypeId;
	protected String name;
	protected Map<SoilType,Map<ContextTypeStatus,Double>> adjustmentFactors;
	
	public ContextType(Short contextTypeId, String name, Map<SoilType,Map<ContextTypeStatus,Double>> adjustmentFactors) {
		this.contextTypeId = contextTypeId;
		this.name = name;
		this.adjustmentFactors = adjustmentFactors;
	}
	
	public Short getContextTypeId() {
		return this.contextTypeId;
	}
	public String getName() {
		return this.name;
	}
	
	public Double getAdjustmentFactor(SoilType soilType, ContextTypeStatus status) {
		return this.adjustmentFactors.get(soilType).get(status);
	}
	
	public Double getStepAdjustmentFactor(SoilType soilType, ContextTypeStatus expectedContextStatus, ContextTypeStatus actualContextStatus) {
		Double factor = 0.0;
		if (! expectedContextStatus.equals(actualContextStatus)) {
			Double actualFactor = 1.0 + getAdjustmentFactor(soilType, actualContextStatus);
			Double expectedFactor = 1.0 + getAdjustmentFactor(soilType, expectedContextStatus);
			
			factor = (actualFactor / expectedFactor) - 1.0;
		}
		
		return factor;
	}
		
	public boolean equals(Object o) {
		if (o instanceof ContextType) {
			ContextType status = (ContextType) o;
			return this.contextTypeId.equals(status.contextTypeId);
		}
		
		return false;
	}
	
	public int hashCode() {
		return this.contextTypeId.hashCode();
	}
}