package fiuba.lamm.agromodel.decision;

import fiuba.lamm.agromodel.agents.*;
import fiuba.lamm.agromodel.utils.*;
import java.util.*;
import repast.simphony.space.graph.*;

/*
 * Vamos a utilizar 3 redes para la imitacion:
 * 
 * 1. Red vecinal (primeros y segundos vecinos)
 * 2. Red profesional (primeros contactos) (#)
 * 3. Red AACREA (miembros del mismo grupo) (#)
 * 
 * (#) Considerar los campos operados por ese productor que pertenezcan a la misma region del
 *     campo para el cual se evalua la decision
 * 
 */
public class MarginWeightedImitationSelectionMechanism extends LandUseSelectionMechanism {
	public MarginWeightedImitationSelectionMechanism() {
		this.landUseSelectionMechanismId = Type.MarginWeightedImitationSelectionMechanism;
	}
	
	/* 
	 * Hace una seleccion entres las opciones de sus pares teniendo en cuenta los siguientes criterios:
	 * 
	 * 1) Consideramos pares a aquellos campos que tienen un AMP admitido por el farmer y que ademas obtuvieron
	 *    un resultado mejor o igual al nuestro.
	 *    
	 * 2) De los pares candidatos, se efectua una eleccion random entre ellos donde la probabilidad de salir seleccionado es
	 *    proporcional al margen bruto global por hectarea. Si no hubiera pares para imitar, la eleccion es random.
	 */
	public ActivityManagementProportion search(Farm farm, Map<LandUseSelectionRestriction, Object> restrictions) {
		// 1. Peer selection
		// 1a. Obtencion de los posibles AMPs
		List<ActivityManagementProportion> possibleAMPs = getAMPFilteredList(restrictions);
		
		// 1b. Obtencion de los vecinos e inicializacion de variables para el calculo de ggm/ha de ellos
		Manager manager = Manager.getInstance();
		Network<Agent> network = manager.getNeighbourhood();
		Double farmGGMPerHa = farm.getLastGlobalGrossMargin() / farm.getFarmSize();
		Map<Farm, Double> probabilityDistribution = new HashMap<Farm, Double>();
		
		Double totalGGMPerHa = 0.0;
		Iterator<Agent> adjacents = network.getAdjacent(farm).iterator();
		while (adjacents.hasNext()) {
			Farm peer = (Farm) adjacents.next();
			
			// Si el AMP de mi vecino esta admitido en mi lista de posibles AMP, entonces lo que hizo mi par es un posible candidato.
			if (possibleAMPs.contains(peer.getCurrentActivityManagementProportion())) {
				Double peerGGMPerHa = peer.getLastGlobalGrossMargin() / peer.getFarmSize();
				
				// Al vecino lo tengo en cuenta solo si le fue mejor a igual que a mi
				if (peerGGMPerHa >= farmGGMPerHa) {
					totalGGMPerHa += peerGGMPerHa;				
					probabilityDistribution.put(peer, peerGGMPerHa);
				}
			}
		}
		
		if (! probabilityDistribution.isEmpty()) {
			// Hay al menos un vecino al cual imitar
			for (Farm f: probabilityDistribution.keySet()) {
				Double p = probabilityDistribution.get(f);
				probabilityDistribution.put(f, p / totalGGMPerHa);
			}
		
			// 1c. Elijo al azar uno de ellos
			Randomizer randomizer = manager.getScenario().getRandomizer();
			Farm selectedFarm = (Farm) randomizer.getRandomEvent(probabilityDistribution);
			
			// 2. Devuelvo el AMP del campo seleccionado. Seteo el mecanismo definitivo de seleccion
			this.finalLandUseSelectionMechanism = this;
			return selectedFarm.getCurrentActivityManagementProportion();
		} else {
			// Solo me puedo imitar a mi mismo. No tiene sentido. Elijo random
			LandUseSelectionMechanism defaultMechanism = new RandomLandUseSelectionMechanism();
			
			this.finalLandUseSelectionMechanism = defaultMechanism;			
			return defaultMechanism.search(farm, restrictions);
		}
	}
}