package fiuba.lamm.agromodel.decision;

import fiuba.lamm.agromodel.agents.*;
import java.util.*;

public class NOutOfMSearchTriggeringMechanism extends SearchTriggeringMechanism {
	protected Integer minUnsatisfiedCycles;
	protected Integer cyclesInMemory;
	
	protected NOutOfMSearchTriggeringMechanism(Integer minUnsatisfiedCycles, Integer cyclesInMemory) {
		this.minUnsatisfiedCycles = minUnsatisfiedCycles;
		this.cyclesInMemory = cyclesInMemory;
	}
	
	public boolean triggerSearch(Farm farm, Map<LandUseSelectionRestriction, Object> restrictions) {
		// 1. Check using if current AMP is still a candidate. If not, a search must be triggered.
		if (! isAMPStillValid(farm, restrictions)) {
			// Trigger search
			return true;
		}
		
		// 2. Check whether N out of the last M campaigns were satisfactory or not
		boolean triggerSearch = true;
		AspirationLevel aspirationLevel = farm.getAspirationLevel();
		List<Double> globalGrossMargins = aspirationLevel.getGlobalGrossMargins();
		List<Double> aspirationLevels = aspirationLevel.getAspirationLevelsAdjustedByContextComparison();
		List<ActivityManagementProportion> activityManagementProportions = aspirationLevel.getActivityManagementProportions();
		
		ActivityManagementProportion lastProportion = null;
		if (! globalGrossMargins.isEmpty()) {
			int unsatisfiedCroppingCycles = 0;
			int startIndex = (activityManagementProportions.size() < this.cyclesInMemory) ? 0 : ((activityManagementProportions.size() - this.cyclesInMemory));
			lastProportion = activityManagementProportions.get(activityManagementProportions.size() - 1);
			for (int i = startIndex; i < activityManagementProportions.size(); i++) {
				ActivityManagementProportion proportion = activityManagementProportions.get(i);
				if (proportion.equals(lastProportion)) {
					Double level = aspirationLevels.get(i);
					Double globalGrossMargin = aspirationLevels.get(i);
					if (level > globalGrossMargin) unsatisfiedCroppingCycles++;
				}
			}
			
			triggerSearch = (unsatisfiedCroppingCycles >= this.minUnsatisfiedCycles);
		}
		
		return triggerSearch;
	}
}
