package fiuba.lamm.agromodel.decision;

import java.util.*;
import fiuba.lamm.agromodel.*;
import fiuba.lamm.agromodel.agents.*;
import fiuba.lamm.agromodel.execution.*;

public abstract class PlotAssignmentMechanism {
	public abstract void assignLandUseToPlots(Farm farm);
	
	protected void runIntegrityChecks(Farm farm) {
		// Integrity checks
		List<Plot> plots = farm.getPlots();
		ActivityManagementProportion currentAmp = new ActivityManagementProportion(plots);
		ActivityManagementProportion expectedAmp = farm.getNextAMP();
		if (! currentAmp.equals(expectedAmp)) {
			throw new FulminantError("farmId = " + farm.getFarmId() + "; Expected AMP (" + expectedAmp + ") is not equal to current AMP (" + currentAmp + ")");
		}
		
		Region region = farm.getRegion();
		for (Plot plot: plots) {
			ActivityManagement management = plot.getCurrentActivityManagement();
			if (! management.getRegion().equals(region)) {
				throw new FulminantError("Cropping cycle = " + Manager.getInstance().getCurrentCroppingCycle().getCroppingCycleName() + "; plotName = " + plot.getPlotName() + "; current management (" + management.getActivityMgmtName() + ") cannot be used in farm's region (farm region = " + region.getRegionName() + "; management region = " + management.getRegion().getRegionName() + ")");
			}
		}		
	}
}