package fiuba.lamm.agromodel.decision;

import fiuba.lamm.agromodel.agents.*;
import java.util.*;

public abstract class SearchTriggeringMechanism {
	public enum Type { 
		RandomSearchTriggeringMechanism	(0), 
		NOutOfMSearchTriggeringMechanism	(1);
	
		private int id;
		Type(int id) {
			this.id = id;
		}
		
		public static Type byId(int id) {
			Type type = null;
			int i = 0;
			Type types[] = Type.values();
			
			while ((i < types.length) && (type == null)) {
				if (types[i].id == id) type = types[i];
				i++;
			}
			
			return type;
		}
	};
	
	public enum Parameter {
		ProbabilityOfSearching,
		MinUnsatisfiedCycles,
		CyclesInMemory
	}
		
	
	/**
	 * @param searchTriggeringMechanismId
	 * @return	The search triggering mechanism specified by the id
	 */
	public static SearchTriggeringMechanism getInstance(int searchTriggeringMechanismId, Map<SearchTriggeringMechanism.Parameter, Object> mechanismParameters) {
		Type type = Type.byId(searchTriggeringMechanismId);
		if (type.equals(Type.RandomSearchTriggeringMechanism)) {
			return new RandomSearchTriggeringMechanism((Double) mechanismParameters.get(Parameter.ProbabilityOfSearching));
		} else if (type.equals(Type.NOutOfMSearchTriggeringMechanism)) {
			return new NOutOfMSearchTriggeringMechanism((Integer) mechanismParameters.get(Parameter.MinUnsatisfiedCycles), (Integer) mechanismParameters.get(Parameter.CyclesInMemory));
		}
		
		return null;
	}
	
	/**
	 * @param farm	The farm that is being tested
	 * @return		True if the search must be triggered. False if the farm should use the same AMP (Habit mechanism)
	 */
	public abstract boolean triggerSearch(Farm farm, Map<LandUseSelectionRestriction, Object> restrictions);
	
	protected boolean isAMPStillValid(Farm farm, Map<LandUseSelectionRestriction, Object> restrictions) {
		ActivityManagementProportion currentAMP = farm.getCurrentActivityManagementProportion();
		LandUseSelectionMechanism lusm = farm.getOperator().getLandUseSelectionMechanism();
		List<ActivityManagementProportion> possibleAMPs = lusm.getAMPFilteredList(restrictions);
		return possibleAMPs.contains(currentAMP);
	}
}
