package fiuba.lamm.agromodel.decision.factories;

import java.sql.*;
import java.util.*;

import fiuba.lamm.agromodel.*;
import fiuba.lamm.agromodel.decision.*;
import fiuba.lamm.agromodel.execution.*;
import fiuba.lamm.agromodel.factories.*;

public class ContextTypeFactory extends Factory {
	protected static Map<Scenario,ContextTypeFactory> factories;	
	protected Map<Short, ContextType> cache;
	
	public static synchronized ContextTypeFactory getInstance(Scenario scenario) throws SQLException {
		if (factories == null) {
			factories = new TreeMap<Scenario,ContextTypeFactory>();			
		}
		if (! factories.containsKey(scenario)) { 
			factories.put(scenario, new ContextTypeFactory(scenario));
		}
		
		return factories.get(scenario);
	}
	
	protected ContextTypeFactory(Scenario scenario) throws SQLException {
		super();
		
		this.cache = new TreeMap<Short, ContextType>();
		Connection connection = null;
		
		List<SoilType> soilTypes = SoilTypeFactory.getInstance(scenario).findAll();
		List<ContextTypeStatus> contextTypeStatus = ContextTypeStatusFactory.getInstance().findAll();		
		try {
			connection = getConnection();
					
			PreparedStatement st = connection.prepareStatement("select * from context_type");
			PreparedStatement stAdjustmentFactor = connection.prepareStatement("select adjustmentFactor from context_type_adjustment where contextTypeId = ? and soilTypeId = ? and contextTypeStatusId = ?");
			
			ResultSet rs = st.executeQuery();
			while (rs.next()) {
				Short contextTypeId = rs.getShort("contextTypeId");
				Map<SoilType,Map<ContextTypeStatus,Double>> adjustmentFactors = new HashMap<SoilType,Map<ContextTypeStatus,Double>>();
				
				for (SoilType soilType: soilTypes) {
					Map<ContextTypeStatus,Double> soilTypeFactors = new HashMap<ContextTypeStatus,Double>();
					
					for (ContextTypeStatus status: contextTypeStatus) {
						stAdjustmentFactor.setShort(1, contextTypeId);
						stAdjustmentFactor.setByte(2, soilType.getSoilTypeId());
						stAdjustmentFactor.setShort(3, status.getContextTypeStatusId());
						
						ResultSet rsAdjustmentFactor = stAdjustmentFactor.executeQuery();
						if (rsAdjustmentFactor.next()) {
							Double adjustmentFactor = rsAdjustmentFactor.getDouble("adjustmentFactor");
							soilTypeFactors.put(status, adjustmentFactor);
						} else {
							throw new FulminantError("Error al buscar el factor de ajuste para context type id = " + contextTypeId + "; soil type = " + soilType.getSoilTypeName() + "; status = " + status.getName());
						}
						rsAdjustmentFactor.close();
					}
					
					adjustmentFactors.put(soilType, soilTypeFactors);
				}
				
				cache.put(contextTypeId, new ContextType(
						contextTypeId,
						rs.getString("name"),
						adjustmentFactors
					)
				);
			}
			
			rs.close();
			st.close();
			stAdjustmentFactor.close();
		} catch (SQLException sqle) {
			throw new FulminantError("Error buscar los context types: " + sqle);			
		} finally {
			if (connection != null) {
				try {
					connection.close();
				} catch (SQLException sqle) {
					throw new FulminantError("Error buscar los context types: " + sqle);
				}
			}
		}
	}
	
	public ContextType findByPrimaryKey(Short contextTypeId) {
		return cache.get(contextTypeId);
	}
	
	public List<ContextType> findAll() {
		return new ArrayList<ContextType>(cache.values());
	}
	
	public static void clearScenarioCache(Scenario scenario) {
		factories.remove(scenario);
	}
}