package fiuba.lamm.agromodel.decision.factories;

import java.sql.*;
import java.util.*;
import fiuba.lamm.agromodel.decision.*;
import fiuba.lamm.agromodel.execution.*;
import fiuba.lamm.agromodel.factories.*;

public class ContextTypeStatusFactory extends Factory {
	protected Map<Short, ContextTypeStatus> cache;
	protected static ContextTypeStatusFactory uniqueFactory;
	
	public static synchronized ContextTypeStatusFactory getInstance() {
		if (uniqueFactory == null) {
			uniqueFactory = new ContextTypeStatusFactory();
		}
		
		return uniqueFactory;
	}
	
	protected ContextTypeStatusFactory() {
		super();
		
		this.cache = new TreeMap<Short, ContextTypeStatus>();
		Connection connection = null;
		
		try {
			connection = getConnection();
			
			PreparedStatement st = connection.prepareStatement("select * from context_type_status");
			ResultSet rs = st.executeQuery();
			while (rs.next()) {
				cache.put(rs.getShort("contextTypeStatusId"), new ContextTypeStatus(
						rs.getShort("contextTypeStatusId"),
						rs.getString("name")
					)
				);
			}
			rs.close();
			st.close();
		} catch (SQLException sqle) {
			throw new FulminantError("Error al buscar los context type values: " + sqle);			
		} finally {
			if (connection != null) {
				try {
					connection.close();
				} catch (SQLException sqle) {
					throw new FulminantError("Error al buscar los context type values: " + sqle);
				}
			}
		}
	}
	
	public ContextTypeStatus findByPrimaryKey(Short contextTypeStatusId) {
		return cache.get(contextTypeStatusId);
	}
	
	public List<ContextTypeStatus> findAll() {
		return new ArrayList<ContextTypeStatus>(cache.values());
	}
}