package fiuba.lamm.agromodel.decision.factories;

import java.sql.*;
import java.util.*;
import fiuba.lamm.agromodel.decision.*;
import fiuba.lamm.agromodel.execution.*;
import fiuba.lamm.agromodel.factories.*;

public class RotatorTypeFactory extends Factory {
	protected Map<Short, RotatorType> cache;
	protected static RotatorTypeFactory uniqueFactory;
	
	public static synchronized RotatorTypeFactory getInstance() {
		if (uniqueFactory == null) {
			uniqueFactory = new RotatorTypeFactory();
		}
		
		return uniqueFactory;
	}
	
	protected RotatorTypeFactory() {
		super();
		
		this.cache = new TreeMap<Short, RotatorType>();
		Connection connection = null;
		
		try {
			connection = getConnection();
			
			PreparedStatement st = connection.prepareStatement("select * from rotatortype");
			ResultSet rs = st.executeQuery();
			while (rs.next()) {
				cache.put(rs.getShort("rotatorTypeId"), new RotatorType(
						rs.getShort("rotatorTypeId"),
						rs.getString("rotatorTypeName"),
						rs.getInt("minPlotsPerActivity"),
						rs.getInt("maxPlotsPerActivity")
					)
				);
			}
			rs.close();
			st.close();
		} catch (SQLException sqle) {
			throw new FulminantError("Error finding rotator types: " + sqle);			
		} finally {
			if (connection != null) {
				try {
					connection.close();
				} catch (SQLException sqle) {
					throw new FulminantError("Error finding rotator types: " + sqle);
				}
			}
		}
	}
	
	public RotatorType findByPrimaryKey(Short rotatorTypeId) {
		return cache.get(rotatorTypeId);
	}
	
	public List<RotatorType> findAll() {
		return new ArrayList<RotatorType>(cache.values());
	}
}