package fiuba.lamm.agromodel.diffusion;

import fiuba.lamm.agromodel.*;
import fiuba.lamm.agromodel.agents.*;
import fiuba.lamm.agromodel.diffusion.ising.IsingComponentsCalculationMechanism;
import fiuba.lamm.agromodel.execution.*;
import fiuba.lamm.agromodel.utils.*;
import java.util.*;

public abstract class InnovationDiffusionModel {
	public enum Type { 
		RandomInnovationDiffusionModel	(0), 
		IsingInnovationDiffusionModel	(1);
	
		private int id;
		Type(int id) {
			this.id = id;
		}
		
		public static Type byId(int id) {
			Type type = null;
			int i = 0;
			Type types[] = Type.values();
			
			while ((i < types.length) && (type == null)) {
				if (types[i].id == id) type = types[i];
				i++;
			}
			
			return type;
		}
	};
	
	public enum Parameter {
		IsingComponentsCalculationMechanismId,
		RandomAdoptionProbability
	}
	
	public static InnovationDiffusionModel getInstance(Scenario scenario, int innovationDiffusionModelId, Map<InnovationDiffusionModel.Parameter,Object> parameters) {
		Type type = Type.byId(innovationDiffusionModelId);
		InnovationDiffusionModel model = null;
		
		if (type.equals(Type.RandomInnovationDiffusionModel)) {
			Double adoptionProbability = (Double) parameters.get(Parameter.RandomAdoptionProbability);
			model = new RandomInnovationDiffusionModel(adoptionProbability);
		} else if (type.equals(Type.IsingInnovationDiffusionModel)) {			
			Double temperature = (Double) scenario.getParameter("InnovationDiffusion.Temperature");
			Integer croppingCyclesForUtilityCalculation = (Integer) scenario.getParameter("InnovationDiffusion.CroppingCyclesForUtilityCalculation");
			Integer isingComponentsCalculationMechanismId = ((Long) parameters.get(Parameter.IsingComponentsCalculationMechanismId)).intValue();
			IsingComponentsCalculationMechanism isccm = IsingComponentsCalculationMechanism.getInstance(isingComponentsCalculationMechanismId);
			model = new IsingDiffusionModel(temperature, croppingCyclesForUtilityCalculation, isccm);
		} else {
			throw new FulminantError("Unknown InnovationDiffusionModel: " + innovationDiffusionModelId);
		}
		
		return model;
	}
	
	public abstract boolean doesInnovate(Agent agent, ActivityManagement management);
	
	protected boolean getRandomChoice(Randomizer randomizer, Double adoptionProbability) {
		Map<Boolean,Double> distribution = new HashMap<Boolean,Double>();
		distribution.put(new Boolean(true), adoptionProbability);
		distribution.put(new Boolean(false), 1.0 - adoptionProbability);
		return randomizer.getRandomEvent(distribution);
	}
}