package fiuba.lamm.agromodel.diffusion;

import fiuba.lamm.agromodel.*;
import fiuba.lamm.agromodel.agents.*;
import fiuba.lamm.agromodel.diffusion.ising.*;
import fiuba.lamm.agromodel.execution.*;
import fiuba.lamm.agromodel.utils.*;

public class IsingDiffusionModel extends InnovationDiffusionModel {
	protected Double temperature;
	protected Integer croppingCyclesForUtilityCalculation;
	protected IsingComponentsCalculationMechanism isingComponentsCalculationMechanism;
	
	public IsingDiffusionModel(Double temperature, Integer croppingCyclesForUtilityCalculation, IsingComponentsCalculationMechanism isingComponentsCalculationMechanism) {
		this.temperature = temperature;
		this.croppingCyclesForUtilityCalculation = croppingCyclesForUtilityCalculation;
		this.isingComponentsCalculationMechanism = isingComponentsCalculationMechanism;
	}
	
	public boolean doesInnovate(Agent agent, ActivityManagement management) {
		Scenario scenario = Manager.getInstance().getScenario();
				
		/* 1. Calculation of du and dv */
		Double innovationUtility = null;
		Double innovationSocialPressure = null;
		Double alpha = null;
		if (agent instanceof Farm) {
			Farm farm = (Farm) agent;
			alpha = farm.getOperator().getInovatorType().getAlpha();			
		
			innovationUtility = this.isingComponentsCalculationMechanism.getInnovationUtility(farm, management, this.croppingCyclesForUtilityCalculation);
			innovationSocialPressure = this.isingComponentsCalculationMechanism.getInnovationSocialPressure(farm, management);
		} else if (agent instanceof Advisor) {
			if (scenario.hasAttribute(Scenario.Attribute.UseAdvisorsInDiffusionProcess)) {
					// TODO: Implementar la innovacion asesores
			}
		} else {
			throw new Error("Invalid agent type: " + agent.getClass().getName());
		}
		
		/* 2. Calculation of p */
		Double m = alpha * innovationSocialPressure + (1.0 - alpha) * innovationUtility;
		Double p = null;
		if (temperature > 0.0) {
			p = 1.0 / (1.0 + Math.exp((-2.0 * m) / this.temperature));
		} else {
			Double signum = Math.signum(m);
			if (signum == 1.0) {
				p = 1.0;
			} else if (signum == 0.0) {				
				p = 0.5;
			} else {
				p = 0.0;
			}
		}
		
		/* 3. return decision */
		Randomizer randomizer = scenario.getRandomizer();
		return getRandomChoice(randomizer, p);
	}
}