package fiuba.lamm.agromodel.diffusion.advisory.factories;

import java.sql.*;
import java.util.*;
import fiuba.lamm.agromodel.agents.*;
import fiuba.lamm.agromodel.agents.factories.*;
import fiuba.lamm.agromodel.diffusion.advisory.*;
import fiuba.lamm.agromodel.execution.*;

public class StaticAdvisoryModelFactory extends AdvisoryModelFactory {
	@Override
	public void setup(AdvisoryModel advisoryModel, List<Farm> farms) {
		Map<Short,List<Advisor>> farmAdvisors = new HashMap<Short,List<Advisor>>();		
		Manager manager = Manager.getInstance();
		
		Connection connection = null;
		try {
			connection = getConnection();
			AdvisorFactory advisorFactory = AdvisorFactory.getInstance(manager.getScenario());
			PreparedStatement st = connection.prepareStatement("select farmId, advisorId from advisor_farm");
			st.setShort(1, manager.getScenario().getScenarioId());
			
			ResultSet rs = st.executeQuery();
			while (rs.next()) {
				Short farmId = rs.getShort("farmId");
				Advisor advisor = advisorFactory.findByPrimaryKey(rs.getShort("asvisorId"));
				List<Advisor> currentAdvisors = farmAdvisors.get(farmId);
				if (currentAdvisors != null) {
					currentAdvisors.add(advisor);
				} else {
					currentAdvisors = new ArrayList<Advisor>();
				}
				
				farmAdvisors.put(farmId, currentAdvisors);
			}
			
			rs.close();
			st.close();
		} catch (SQLException sqle) {
			throw new FulminantError("Error finding the initial advisors for the static assignment model: " + sqle);
		} finally {
			if (connection != null) {
				try {
					connection.close();
				} catch (SQLException sqle) {
					throw new FulminantError("Error closing the connection to the database: " + sqle);
				}
			}
		}
		
		for (Farm farm: farms) {
			List<Advisor> advisors = farmAdvisors.get(farm.getFarmId());
			if (advisors != null) {
				farm.setAdvisors(advisors);
			}
		}
	}

	@Override
	public List<?> findAll() {
		throw new UnsupportedOperationException("StaticAdvisoryModelFactory does not support this method");
	}
}
