package fiuba.lamm.agromodel.diffusion.ising;

import java.util.*;

import fiuba.lamm.agromodel.*;
import fiuba.lamm.agromodel.agents.*;
import fiuba.lamm.agromodel.execution.*;
import repast.simphony.space.graph.*;

public abstract class IsingComponentsCalculationMechanism {
	public enum Type { 
		UtilityBasedIsingComponentsCalculationMechanism	(0), 
		ImitationBasedComponentsCalculationMechanism	(1);
	
		private int id;
		Type(int id) {
			this.id = id;
		}
		
		public static Type byId(int id) {
			Type type = null;
			int i = 0;
			Type types[] = Type.values();
			
			while ((i < types.length) && (type == null)) {
				if (types[i].id == id) type = types[i];
				i++;
			}
			
			return type;
		}
	};
	
	public static IsingComponentsCalculationMechanism getInstance(int isingComponentsCalculationMechanism) {
		Type type = Type.byId(isingComponentsCalculationMechanism);
		IsingComponentsCalculationMechanism mechanism = null;
		
		if (type.equals(Type.UtilityBasedIsingComponentsCalculationMechanism)) {
			mechanism = new UtilityBasedIsingComponentsCalculationMechanism();
		} else if (type.equals(Type.ImitationBasedComponentsCalculationMechanism)) {			
			mechanism = new ImitationBasedIsingComponentsCalculationMechanism();
		} else {
			throw new FulminantError("Unknown IsingComponentsCalculationMechanism: " + isingComponentsCalculationMechanism);
		}
		
		return mechanism;
	}
	
	// Calculo de du
	public abstract Double getInnovationUtility(Farm farm, ActivityManagement management, Integer croppingCyclesForUtilityCalculation);
		
	/**
	 * La presion social se calcula para aquellos operadores que no implementen seleccion de uso de la tierra por habito o random.
	 * En estos casos, la presion social se considera 0 para que el modelo de Ising sea estocastico al momento de elegir la opcion.
	 */
	public Double getInnovationSocialPressure(Farm farm, ActivityManagement management) {
		// TODO: Por el momento existe la red de vecinos, donde cada uno pesa igual.
		// 1. Presion social de vecinos fisicos		
		int numberOfAdopters = 0;
		int numberOfPeers = 0;
		Network<Agent> network = Manager.getInstance().getNeighbourhood();
		Iterator<Agent> adjacents = network.getAdjacent(farm).iterator();
		while (adjacents.hasNext()) {
			Farm peer = (Farm) adjacents.next();
			if (! peer.getUnknownManagements().contains(management)) {
				numberOfAdopters++;
			}
			
			numberOfPeers++;
		}
		
		// TODO: Implementar presion social de asesores y red de AACREA
		Double socialPressure = ((2.0 * (double) numberOfAdopters) / (double) numberOfPeers) - 1.0;
		return socialPressure;
	}	
}
