package fiuba.lamm.agromodel.execution;

import java.io.*;

public class CommonTablesLoaderHandler implements ExecutionHandler {
	public void execute() throws ExecutionException {
		// 1. Find script
		File projectRoot = new File(System.getenv("PROJECT_ROOT"));
		File inputDir = new File(projectRoot, "input");
		if (! inputDir.isDirectory()) {
			throw new ExecutionException("Input directory not found: " + inputDir.getAbsolutePath());
		}
		
		File scriptsDir = new File(inputDir, "Scripts");
		if (! scriptsDir.isDirectory()) {
			throw new ExecutionException("Scripts directory not found: " + scriptsDir.getAbsolutePath());
		}
		
		File commonTablesScript = new File(scriptsDir, "loadCommonTables.pl");
		if (! commonTablesScript.isFile()) {
			throw new ExecutionException("Common tables loader not found: " + commonTablesScript.getAbsolutePath());
		}
		
		// 2. Execute script
		ProcessBuilder pb = new ProcessBuilder(commonTablesScript.getAbsolutePath());
		pb.directory(scriptsDir);
		try {
			Process p = pb.start();
			if (p.waitFor() != 0) {
				BufferedReader reader = new BufferedReader(new InputStreamReader(p.getErrorStream()));
				String message = new String();
				String line = reader.readLine();
				while (line != null) {
					message += line + "\n";
					line = reader.readLine();					
				}
				reader.close();
				
				throw new ExecutionException("Error loading common tables: " + message);
			}
		} catch (IOException ioe) {
			throw new ExecutionException("Error loading common tables: " + ioe.getMessage());
		} catch (InterruptedException ie) {
			throw new ExecutionException("Error loading common tables: " + ie.getMessage());
		}
	}
}
