package fiuba.lamm.agromodel.execution;

import java.util.*;

public class ScenarioEnsemble {
	public enum Attribute {
		ClimateRealizations		(1); // El conjunto de escenarios tiene trayectorias climaticas
		
		private int bit;
		private Attribute(int bit) {
			this.bit = bit;
		}
		
		public int getBit() {
			return this.bit;
		}
		
		public static Map<Attribute,Boolean> getAttributes(int bits) {
			Map<Attribute,Boolean> attributes = new HashMap<Attribute,Boolean>();
			for (Attribute attribute: Attribute.values()) {
				boolean active = ((attribute.bit & bits) != 0);
				attributes.put(attribute, active);
			}
			
			return attributes;
		}
	}
	
	protected Short scenarioEnsembleId;
	protected String name;
	protected Map<Attribute,Boolean> attributes;
	
	public ScenarioEnsemble(Short scenarioEnsembleId, String name, Integer attributes) {
		this.scenarioEnsembleId = scenarioEnsembleId;
		this.name = name;
		this.attributes = Attribute.getAttributes(attributes);
	}
	
	// Accessors
	public Short getScenarioEnsembleId() {
		return this.scenarioEnsembleId;
	}
	public String getName() {
		return this.name;
	}
	public Boolean hasAttribute(ScenarioEnsemble.Attribute attribute) {
		return this.attributes.get(attribute);
	}
	
	public int hashCode() {
		return this.scenarioEnsembleId.hashCode();
	}
}