package fiuba.lamm.agromodel.execution;

import java.io.*;
import java.util.*;

public class ScenarioLoaderHandler implements ExecutionHandler {
	protected List<Short> scenarioId;

	public ScenarioLoaderHandler(List<Short> scenarioId) {
		this.scenarioId = scenarioId;
	}
	
	public void execute() throws ExecutionException {
		// 1. Find script
		File projectRoot = new File(System.getenv("PROJECT_ROOT"));
		File inputDir = new File(projectRoot, "input");
		if (! inputDir.isDirectory()) {
			throw new ExecutionException("Input directory not found: " + inputDir.getAbsolutePath());
		}
		
		File scriptsDir = new File(inputDir, "Scripts");
		if (! scriptsDir.isDirectory()) {
			throw new ExecutionException("Scripts directory not found: " + scriptsDir.getAbsolutePath());
		}
		
		File scenarioScript = new File(scriptsDir, "loadScenarios.pl");
		if (! scenarioScript.isFile()) {
			throw new ExecutionException("Scenario loader not found: " + scenarioScript.getAbsolutePath());
		}
		
		// 2. Execute script for all scenarios
		String scenarioIdString = this.scenarioId.get(0).toString();
		for (int i = 1; i < this.scenarioId.size(); i++) {
			scenarioIdString += "," + this.scenarioId.get(i).toString();
		}
		
		ProcessBuilder pb = new ProcessBuilder(scenarioScript.getAbsolutePath(), scenarioIdString);
		pb.directory(scriptsDir);
		try {
			Process p = pb.start();
			if (p.waitFor() != 0) {
				BufferedReader reader = new BufferedReader(new InputStreamReader(p.getErrorStream()));
				String message = new String();
				String line = reader.readLine();
				while (line != null) {
					message += line + "\n";
					line = reader.readLine();
				}
				reader.close();
				
				throw new ExecutionException("Error loading scenarios " + scenarioIdString + ": " + message);
			}
		} catch (IOException ioe) {
			throw new ExecutionException("Error loading scenarios " + scenarioIdString + ": " + ioe.getMessage());
		} catch (InterruptedException ie) {
			throw new ExecutionException("Error loading scenarios " + scenarioIdString + ": " + ie.getMessage());
		}		
	}
}
