package fiuba.lamm.agromodel.execution.ensemble;

import fiuba.lamm.agromodel.execution.*;
import java.io.*;
import java.util.*;
import javax.mail.*;
import javax.mail.internet.*;

public class NotificationHandler extends Authenticator implements ExecutionHandler {
	protected Short scenarioEnsembleId;
	protected String uploadUrl;
	protected String recipientUser;
	protected String recipientEmail;
	protected Properties env;
	protected String errorMessage;
	
	public NotificationHandler(String uploadUrl, String recipientUser, String recipientEmail, Short scenarioEnsembleId) {
		fiuba.lamm.agromodel.utils.System.init();
		this.env = System.getProperties();		
		this.uploadUrl = uploadUrl;
		this.recipientUser = recipientUser;
		this.recipientEmail = recipientEmail;
		this.scenarioEnsembleId = scenarioEnsembleId;
		this.errorMessage = null;
	}

	protected PasswordAuthentication getPasswordAuthentication() {
        return new PasswordAuthentication(env.getProperty("fiuba.lamm.agromodel.mail.smtp.user"), env.getProperty("fiuba.lamm.agromodel.mail.smtp.password"));
	}
	
	public void setErrorMessage(String errorMessage) {
		this.errorMessage = errorMessage;
	}
	
	public void execute() throws ExecutionException {
		// 1. Initialize session
		Properties props = new Properties();
		props.setProperty("mail.transport.protocol", "smtp");
		props.setProperty("mail.host", env.getProperty("fiuba.lamm.agromodel.mail.smtp.host"));
		props.put("mail.smtp.auth", "true");
		props.put("mail.smtp.starttls.enable", "true");
		props.put("mail.smtp.port", env.getProperty("fiuba.lamm.agromodel.mail.smtp.port"));
		props.put("mail.smtp.socketFactory.port", env.getProperty("fiuba.lamm.agromodel.mail.smtp.port"));
		props.put("mail.smtp.socketFactory.class", "javax.net.ssl.SSLSocketFactory");
		props.put("mail.smtp.socketFactory.fallback", "false");
		props.setProperty("mail.smtp.quitwait", "false");		
		Session session = Session.getInstance(props, this);
		
		try {
			// 2. Build message
			MimeMessage message = new MimeMessage(session);
			message.setFrom(new InternetAddress(env.getProperty("fiuba.lamm.agromodel.mail.smtp.user"), "Distributed executor mailer"));
            message.addRecipient(Message.RecipientType.TO, new InternetAddress(recipientEmail, recipientUser));
            message.setSubject("Execution status for ensemble: " + this.scenarioEnsembleId);
            message.setSentDate(new Date());
            
            String content = null;
            if (this.errorMessage != null) {
            	content = "Execution failed with this message: " + this.errorMessage + "\n\nNo results will be committed.";
            } else {
            	if (this.uploadUrl != null) {
            		content = "Execution was successful! Results have been automatically committed to repository.\n\nGo to this url to view the output files: " + this.uploadUrl;
            	} else {
            		content = "Execution was successful! Results have been automatically committed to repository.";
            	}
            }
           	message.setContent(content, "text/plain");
           	
           	// 3. Send message
           	Transport.send(message);
		} catch (UnsupportedEncodingException uee) {
			throw new ExecutionException("Recipient address is invalid: " + uee);
		} catch (MessagingException me) {
			throw new ExecutionException("Error sending notification e-mail: " + me);
		}
	}
}