package fiuba.lamm.agromodel.execution.ensemble;

import fiuba.lamm.agromodel.execution.*;
import java.io.*;

public class ScenarioEnsembleLoaderHandler implements ExecutionHandler {
	protected Short scenarioEnsembleId;

	public ScenarioEnsembleLoaderHandler(Short scenarioEnsembleId) {
		this.scenarioEnsembleId = scenarioEnsembleId;
	}
	
	public void execute() throws ExecutionException {
		// 1. Find script
		File projectRoot = new File(System.getenv("PROJECT_ROOT"));
		File inputDir = new File(projectRoot, "input");
		if (! inputDir.isDirectory()) {
			throw new ExecutionException("Input directory not found: " + inputDir.getAbsolutePath());
		}
		
		File scriptsDir = new File(inputDir, "Scripts");
		if (! scriptsDir.isDirectory()) {
			throw new ExecutionException("Scripts directory not found: " + scriptsDir.getAbsolutePath());
		}
		
		File scenarioEnsembleScript = new File(scriptsDir, "loadScenarioEnsemble.pl");
		if (! scenarioEnsembleScript.isFile()) {
			throw new ExecutionException("Scenario loader not found: " + scenarioEnsembleScript.getAbsolutePath());
		}
		
		// 2. Execute script for scenario ensemble
		ProcessBuilder pb = new ProcessBuilder(scenarioEnsembleScript.getAbsolutePath(), this.scenarioEnsembleId.toString());
		pb.directory(scriptsDir);
		try {
			Process p = pb.start();
			if (p.waitFor() != 0) {
				BufferedReader reader = new BufferedReader(new InputStreamReader(p.getErrorStream()));
				String message = new String();
				String line = reader.readLine();
				while (line != null) {
					message += line + "\n";
					line = reader.readLine();
				}
				reader.close();
				
				throw new ExecutionException("Error loading scenario ensemble " + this.scenarioEnsembleId + ": " + message);
			}
		} catch (IOException ioe) {
			throw new ExecutionException("Error loading scenario ensemble " + this.scenarioEnsembleId + ": " + ioe.getMessage());
		} catch (InterruptedException ie) {
			throw new ExecutionException("Error loading scenario ensemble " + this.scenarioEnsembleId + ": " + ie.getMessage());
		}		
	}
}
