package fiuba.lamm.agromodel.execution.factories;

import java.sql.*;
import java.util.*;
import fiuba.lamm.agromodel.execution.*;
import fiuba.lamm.agromodel.factories.*;

public class ScenarioEnsembleFactory extends Factory {
	protected Map<Short,ScenarioEnsemble> cache;
	protected static ScenarioEnsembleFactory uniqueFactory;
	
	public static synchronized ScenarioEnsembleFactory getInstance() {
		if (uniqueFactory == null) {
			uniqueFactory = new ScenarioEnsembleFactory();
		}
		
		return uniqueFactory;
	}
	
	protected ScenarioEnsembleFactory() {
		super();		
		this.cache = new TreeMap<Short,ScenarioEnsemble>();
		
		Connection connection = null;
		try {
			connection = getConnection();
			
			PreparedStatement st = connection.prepareStatement("select * from scenario_ensemble");			
			ResultSet rs = st.executeQuery();
			while (rs.next()) {
				Short scenarioEnsembleId = rs.getShort("scenarioEnsembleId");					
								
				ScenarioEnsemble scenarioEnsemble = new ScenarioEnsemble(
					scenarioEnsembleId,
					rs.getString("name"),
					rs.getInt("attributes")					
				);
				
				
				this.cache.put(scenarioEnsembleId, scenarioEnsemble);
			}
			
			rs.close();
			st.close();			
		} catch (SQLException sqle) {
			throw new FulminantError("Error creating ScenarioFactory: " + sqle);
		} finally {
			if (connection != null) {
				try {
					connection.close();
				} catch (SQLException sqle) {
					throw new FulminantError("Error closing connection: " + sqle);
				}
			}
		}
	}
	
	public ScenarioEnsemble findByPrimaryKey(Short scenarioGroupId) {
		return this.cache.get(scenarioGroupId);
	}
	
	public List<?> findAll() {
		return new ArrayList<ScenarioEnsemble>(this.cache.values());
	}
}