package fiuba.lamm.agromodel.execution.factories;

import java.lang.reflect.*;
import java.sql.*;
import java.util.*;
import fiuba.lamm.agromodel.execution.*;
import fiuba.lamm.agromodel.factories.*;

public class ScenarioFactory extends Factory {
	protected Map<Short,Scenario> cacheByPK;
	protected Map<Short, List<Scenario>> cacheByScenarioEnsemble;
	protected static ScenarioFactory uniqueFactory;
	
	public static synchronized ScenarioFactory getInstance() {
		if (uniqueFactory == null) {
			uniqueFactory = new ScenarioFactory();
		}
		
		return uniqueFactory;
	}
	
	protected ScenarioFactory() {
		super();		
		this.cacheByPK = new TreeMap<Short,Scenario>();
		this.cacheByScenarioEnsemble = new TreeMap<Short, List<Scenario>>();
		
		Connection connection = null;
		try {
			connection = getConnection();
			
			PreparedStatement st = connection.prepareStatement("select * from scenario order by scenarioId");
			PreparedStatement stParameters = connection.prepareStatement("select df.name as parameterName, df.type as parameterType, ifnull(sp.parameterValue, df.defaultValue) as parameterValue from parameter df left join scenario_parameter sp on (df.name = sp.parameterName) and (sp.scenarioId = ?)");
			
			ResultSet rs = st.executeQuery();
			while (rs.next()) {
				Short scenarioId = rs.getShort("scenarioId");
				
				// Busco los parametros especificos para ese escenario y los que no esten definidos los saco de la tabla de parametros default
				Map<String,Object> parameters = new HashMap<String,Object>();
				stParameters.setShort(1, scenarioId);
				ResultSet rsParameters = stParameters.executeQuery();
				while (rsParameters.next()) {
					String parameterName = rsParameters.getString("parameterName");
					String parameterType = rsParameters.getString("parameterType");
					String parameterValue = rsParameters.getString("parameterValue");
					if (! rsParameters.wasNull()) {
						Constructor<?> constructor = null;
						try {
							Class<?> parameterClass = Class.forName(parameterType);
							constructor = parameterClass.getConstructor(java.lang.String.class);
						} catch (ClassNotFoundException cnfe) {
							throw new FulminantError("Class not found: " + parameterType);
						} catch (NoSuchMethodException nsme) {
							throw new FulminantError("No such constructor for class " + parameterType + " with argument of class java.lang.String");						
						}
						
						try {
							parameters.put(parameterName, constructor.newInstance(parameterValue));						
						} catch (Exception e) {
							throw new FulminantError("Error instantiating parameter " + parameterName + " (value = " + parameterValue + ")");
						}
					} else {
						throw new FulminantError("Parameter " + parameterName + " is null for scenario " + scenarioId);
					}					
				}
				rsParameters.close();
				
				// Ensamble de escenarios
				Short scenarioEnsembleId = rs.getShort("scenarioEnsembleId");
				ScenarioEnsemble scenarioEnsemble = null;
				if (! rs.wasNull()) {
					scenarioEnsemble = ScenarioEnsembleFactory.getInstance().findByPrimaryKey(scenarioEnsembleId);
				}
								
				Scenario scenario = new Scenario(
					scenarioId,
					rs.getString("name"),
					rs.getString("description"),
					rs.getInt("attributes"),
					parameters,					
					rs.getString("shapeFileName"),
					scenarioEnsemble
				);
				
				
				this.cacheByPK.put(scenarioId, scenario);
				if (scenarioEnsemble != null) {
					if (! this.cacheByScenarioEnsemble.containsKey(scenarioEnsemble.getScenarioEnsembleId())) {
						this.cacheByScenarioEnsemble.put(scenarioEnsembleId, new ArrayList<Scenario>());
					}
					List<Scenario> scenarios = this.cacheByScenarioEnsemble.get(scenarioEnsemble.getScenarioEnsembleId());
					scenarios.add(scenario);
				}
			}
			
			rs.close();
			st.close();
			stParameters.close();
		} catch (SQLException sqle) {
			throw new FulminantError("Error creating ScenarioFactory: " + sqle);
		} finally {
			if (connection != null) {
				try {
					connection.close();
				} catch (SQLException sqle) {
					throw new FulminantError("Error closing connection: " + sqle);
				}
			}
		}
	}
	
	public Scenario findByPrimaryKey(Short scenarioId) {
		return this.cacheByPK.get(scenarioId);
	}
	
	public List<Scenario> findByScenarioEnsembleId(Short scenarioEnsembleId) {
		return this.cacheByScenarioEnsemble.get(scenarioEnsembleId);
	}
	
	public List<?> findAll() {
		return new ArrayList<Scenario>(this.cacheByPK.values());
	}
}
