package fiuba.lamm.agromodel.factories;

import java.sql.*;
import java.util.*;
import fiuba.lamm.agromodel.*;
import fiuba.lamm.agromodel.execution.*;

public class ActivityFactory extends Factory {
	protected Map<Byte, Activity> cache;
	protected static ActivityFactory uniqueFactory;
	public static ActivityFactory getInstance() {
		if (uniqueFactory == null) {
			uniqueFactory = new ActivityFactory();
		}
		
		return uniqueFactory;
	}
	
	protected ActivityFactory() {
		super();
		
		this.cache = new TreeMap<Byte, Activity>();		
		Connection connection = null;		
		try {
			connection = getConnection();
			
			// TODO: Por ahora se buscan solamente actividades relacionadas con la agricultura porque no esta implementada la logica de la ganaderia. Es por eso que se hace un join con agvarcostinfo.
			ActivityGroupFactory activityGroupFactory = ActivityGroupFactory.getInstance();
			PreparedStatement st = connection.prepareStatement("select e.activityId, activityName, activityType, activityUnit, activityGroupId, harvestCostRate, marketingCostRate from activity e join agvarcostinfo a using(activityId)");
			ResultSet rs = st.executeQuery();
			while (rs.next()) {
				ActivityGroup activityGroup = activityGroupFactory.findByPrimaryKey(rs.getByte("activityGroupId"));
				
				cache.put(rs.getByte("activityId"), new Activity(
						rs.getByte("activityId"),
						rs.getString("activityName"),
						rs.getString("activityType"),
						rs.getString("activityUnit"),
						activityGroup,
						rs.getDouble("harvestCostRate"),
						rs.getDouble("marketingCostRate"),
						null // TODO: Agregar rs.getDouble("aggregateCost") cuando se use ganaderia
					)
				);
			}
			rs.close();
			st.close();
		} catch (SQLException sqle) {
			throw new FulminantError("Error al buscar los activities: " + sqle);			
		} finally {
			if (connection != null) {
				try {
					connection.close();
				} catch (SQLException sqle) {
					throw new FulminantError("Error al buscar los activities: " + sqle);
				}
			}
		}
	}
	
	public Activity findByPrimaryKey(Byte activityId) {
		return cache.get(activityId);
	}
	
	public List<Activity> findByActivityGroup(ActivityGroup activityGroup) {
		List<Activity> activities = new ArrayList<Activity>();
		List<Activity> all = findAll();
		
		for (Activity activity: all) {
			if (activity.getActivityGroup().equals(activityGroup)) {
				activities.add(activity);
			}
		}
		
		return activities;
	}
	
	public List<Activity> findAll() {
		return new ArrayList<Activity>(cache.values());
	}
}