package fiuba.lamm.agromodel.factories;

import java.sql.*;
import java.util.*;
import fiuba.lamm.agromodel.*;
import fiuba.lamm.agromodel.agents.factories.*;
import fiuba.lamm.agromodel.execution.*;

public class CreaGroupFactory extends Factory {
	protected static Map<Scenario,CreaGroupFactory> factories;	
	protected Map<Short, CreaGroup> cache;
	
	public static synchronized CreaGroupFactory getInstance(Scenario scenario) throws SQLException {
		if (factories == null) {
			factories = new TreeMap<Scenario,CreaGroupFactory>();			
		}
		if (! factories.containsKey(scenario)) { 
			factories.put(scenario, new CreaGroupFactory(scenario));
		}
		
		return factories.get(scenario);
	}
	
	protected CreaGroupFactory(Scenario scenario) throws SQLException {
		super();		
		this.cache = new TreeMap<Short, CreaGroup>();
	
		CreaRegionFactory creaRegionFactory = CreaRegionFactory.getInstance(scenario);
		AdvisorFactory advisorFactory = AdvisorFactory.getInstance(scenario);
		
		Connection connection = getConnection();
		PreparedStatement st = connection.prepareStatement("select * from creagroup");
		ResultSet rs = st.executeQuery();
		
		while (rs.next()) {
			this.cache.put(rs.getShort("creaGroupId"), new CreaGroup(
					rs.getShort("creaGroupId"),
					rs.getString("creaGroupName"),
					rs.getString("creaGroupCode"),
					creaRegionFactory.findByPrimaryKey(rs.getByte("creaRegionId")),
					advisorFactory.findByPrimaryKey(rs.getShort("advisorId")),
					rs.getInt("groupSize"),
					rs.getDouble("centroidLat"),
					rs.getDouble("centroidLon"),
					rs.getDouble("membershipFee")
				)
			);
		}
		rs.close();
		st.close();
		connection.close();
	}
	
	public CreaGroup findByPrimaryKey(Short creaGroupId) {
		return this.cache.get(creaGroupId);
	}
	
	public static void clearScenarioCache(Scenario scenario) {
		factories.remove(scenario);
	}
	
	public List<CreaGroup> findAll() {
		return new ArrayList<CreaGroup>(this.cache.values());
	}
}