package fiuba.lamm.agromodel.factories;

import java.sql.*;
import java.util.*;
import fiuba.lamm.agromodel.*;
import fiuba.lamm.agromodel.agents.factories.*;
import fiuba.lamm.agromodel.execution.*;

public class CreaRegionFactory extends Factory {
	protected Map<Byte, CreaRegion> cache;
	protected static Map<Scenario,CreaRegionFactory> factories;
	
	public static synchronized CreaRegionFactory getInstance(Scenario scenario) throws SQLException {
		if (factories == null) {
			factories = new TreeMap<Scenario,CreaRegionFactory>();			
		}
		if (! factories.containsKey(scenario)) {
			factories.put(scenario, new CreaRegionFactory(scenario));
		}
		
		return factories.get(scenario);
	}
	
	protected CreaRegionFactory(Scenario scenario) throws SQLException {
		super();		
		this.cache = new TreeMap<Byte, CreaRegion>();
		
		AdvisorFactory advisorFactory = AdvisorFactory.getInstance(scenario);
		Connection connection = getConnection();
		
		try {
			PreparedStatement st = connection.prepareStatement("select * from crearegion");
			ResultSet rs = st.executeQuery();
			while (rs.next()) {
				cache.put(rs.getByte(1), new CreaRegion(
						rs.getByte("creaRegionId"), 
						rs.getString("creaRegionName"),
						rs.getInt("numberOfGroups"),
						rs.getInt("numberOfMembers"),
						advisorFactory.findByPrimaryKey(rs.getShort("coordinatorId"))
					)
				);
			}
			rs.close();
			st.close();
		} catch (SQLException sqle) {
			throw new FulminantError("Error buscar las regiones crea: " + sqle);			
		} finally {
			if (connection != null) {
				try {
					connection.close();
				} catch (SQLException sqle) {
					throw new FulminantError("Error buscar las regiones crea: " + sqle);
				}
			}
		}
	}
	
	public CreaRegion findByPrimaryKey(Byte creaRegionId) throws SQLException {
		return cache.get(creaRegionId);
	}
	
	public static void clearScenarioCache(Scenario scenario) {
		factories.remove(scenario);
	}
	
	public List<CreaRegion> findAll() {
		throw new UnsupportedOperationException("CreaRegionFactory does not support this method");
	}
}