package fiuba.lamm.agromodel.factories;

import java.sql.*;
import java.util.*;
import javax.sql.*;
import fiuba.lamm.agromodel.agents.factories.*;
import fiuba.lamm.agromodel.execution.*;
import repast.simphony.freezedry.datasource.*;

public abstract class Factory {
	protected static DataSource dataSource;
	
	/**
	 * Class constructor. Retrieves parameters in config.properties file to connect to a data source 
	 */
	public Factory() {
		/* Obtiene los datos de los properties del sistema */
		if (dataSource == null) {
			try {
				String url = System.getProperty("fiuba.lamm.agromodel.db.url");
				String userName = System.getProperty("fiuba.lamm.agromodel.db.user");
				String password = System.getProperty("fiuba.lamm.agromodel.db.password");
				String driverName = System.getProperty("fiuba.lamm.agromodel.db.driverClassName");
				
				DBConnectionInfo info = new DBConnectionInfo(userName, password, url, driverName);
				dataSource = DBUtils.getDataSource(info);
			} catch (ClassNotFoundException cnfe) {
				throw new FulminantError("Clase no encontrada: " + cnfe);
			}
		}
	}
	
	/**
	 * Finds a suitable factory for a given class. If no factory is found, null is returned
	 * 
	 * @param clazz			The class whose factory is to be found
	 * @param scenarioId	Scenario Id of the simulation
	 * @return				The factory for the class
	 * @throws				SQLException if exception is raised during connection to data source
	 */
	public static Factory getFactoryForClass(Class<?> clazz, Scenario scenario) throws SQLException {
		if (clazz.equals(fiuba.lamm.agromodel.agents.Farm.class)) {
			return FarmFactory.getInstance(scenario);
		} else if (clazz.equals(fiuba.lamm.agromodel.agents.Farmer.class)) {
			return FarmerFactory.getInstance(scenario);
		} else if (clazz.equals(fiuba.lamm.agromodel.Plot.class)) {
			return FarmFactory.getInstance(scenario).getPlotFactory();
		} else if (clazz.equals(fiuba.lamm.agromodel.SoilType.class)) {
			return SoilTypeFactory.getInstance(scenario);
		}
		
		return null;
	}
	
	/**
	 * Finds all the objects. Optional operation
	 */
	public abstract List<?> findAll();
	
	/**
	 * Retrieves a connection to data source specified in file config.properties
	 * 
	 * @return					Connection to data source
	 * @throws SQLException		If any exception is raised while trying to connect to the data source
	 */
	public Connection getConnection() throws SQLException {
		return dataSource.getConnection();
	}
}