package fiuba.lamm.agromodel.factories;

import java.sql.*;
import java.util.*;
import fiuba.lamm.agromodel.*;
import fiuba.lamm.agromodel.execution.*;

public class InnovatorTypeFactory extends Factory {
	protected Map<Byte, InnovatorType> cache;
	protected static InnovatorTypeFactory uniqueFactory;
	public static InnovatorTypeFactory getInstance() {
		if (uniqueFactory == null) {
			uniqueFactory = new InnovatorTypeFactory();
		}
		
		return uniqueFactory;
	}
	
	protected InnovatorTypeFactory() {
		super();
		
		this.cache = new TreeMap<Byte, InnovatorType>();
		Connection connection = null;
		
		try {
			connection = getConnection();
			
			PreparedStatement st = connection.prepareStatement("select * from innovatortype");
			ResultSet rs = st.executeQuery();
			while (rs.next()) {
				cache.put(rs.getByte("innovatorTypeId"), new InnovatorType(rs.getByte("innovatorTypeId"), rs.getString("innovatorTypeName"), rs.getDouble("defaultProportion"), rs.getDouble("alphaMin"), rs.getDouble("alphaMax")));
			}
			rs.close();
			st.close();
		} catch (SQLException sqle) {
			throw new FulminantError("Error al buscar los tipos de innovadores: " + sqle);			
		} finally {
			if (connection != null) {
				try {
					connection.close();
				} catch (SQLException sqle) {
					throw new FulminantError("Error al cerrar la conexion a la base de datos: " + sqle);
				}
			}
		}
	}
	
	public InnovatorType findByPrimaryKey(Byte innovatorTypeId) {
		return cache.get(innovatorTypeId);
	}
	
	public List<InnovatorType> findAll() {
		return new ArrayList<InnovatorType>(this.cache.values());
	}
}