package fiuba.lamm.agromodel.factories;

import java.sql.*;
import java.util.*;

import fiuba.lamm.agromodel.*;
import fiuba.lamm.agromodel.execution.*;

public class SoilTypeFactory extends Factory {
	protected static Map<Scenario,SoilTypeFactory> factories;	
	protected Map<Byte, SoilType> cache;
	
	public static synchronized SoilTypeFactory getInstance(Scenario scenario) {
		if (factories == null) {
			factories = new TreeMap<Scenario,SoilTypeFactory>();			
		}
		if (! factories.containsKey(scenario)) { 
			try {
				factories.put(scenario, new SoilTypeFactory(scenario));
			} catch (SQLException sqle) {
				throw new FulminantError("Error creating factory for scenario " + scenario);
			}
		}
		
		return factories.get(scenario);
	}
	
	protected SoilTypeFactory(Scenario scenario) throws SQLException {		
		super();		
		this.cache = new TreeMap<Byte, SoilType>();
		
		Connection connection = null;		
		try {
			connection = getConnection();
			
			PreparedStatement st = connection.prepareStatement("select st.* from soiltype st where exists (select * from farm f where f.scenarioId = ? and f.soilType = st.soilTypeId)");
			st.setShort(1, scenario.getScenarioId());
			ResultSet rs = st.executeQuery();
			while (rs.next()) {
				Region region = RegionFactory.getInstance().findByPrimaryKey(rs.getByte("regionId"));
				this.cache.put(rs.getByte("soilTypeId"), new SoilType(rs.getByte("soilTypeId"), region, rs.getString("soilTypeName"), rs.getString("soilTexture"), rs.getDouble("soilTypeProportionOfRegion")));				
			}
			rs.close();
			st.close();
		} catch (SQLException sqle) {
			throw new FulminantError("Error finding activities labors" + sqle);			
		} finally {
			if (connection != null) {
				try {
					connection.close();
				} catch (SQLException sqle) {
					throw new FulminantError("Error finding activities labors" + sqle);
				}
			}
		}
	}
	
	public SoilType findByPrimaryKey(Byte soilTypeId) {
		return cache.get(soilTypeId);
	}
	
	public List<SoilType> findAll() {
		return new ArrayList<SoilType>(cache.values());
	}
	
	public List<SoilType> findByRegion(Region region) {
		List<SoilType> soilTypes = new ArrayList<SoilType>();
		List<SoilType> types = new ArrayList<SoilType>(cache.values());
		
		for (SoilType soilType: types) {
			if (soilType.getRegion().equals(region)) {
				soilTypes.add(soilType);
			}
		}
		
		return soilTypes;
	}
	
	public static void clearScenarioCache(Scenario scenario) {
		factories.remove(scenario);
	}	
}