package fiuba.lamm.agromodel.math.custom;

import java.util.*;
import fiuba.lamm.agromodel.decision.*;
import fiuba.lamm.agromodel.math.*;

public class CPTFitnessFunction extends FitnessFunction {
	protected CPTValueFunction valueFunction;
	protected VectorialField objectiveWeightingFunction;
	protected Double gammaGains;
	protected Double gammaLosses;

	public CPTFitnessFunction(CPTValueFunction valueFunction, VectorialField objectiveWeightingFunction, Double gammaGains, Double gammaLosses) {
		this.valueFunction = valueFunction;		
		this.objectiveWeightingFunction = objectiveWeightingFunction;
		this.gammaGains = gammaGains;
		this.gammaLosses = gammaLosses;
	}
	
	public LandUseSelectionMechanism.Type getLandUseSelectionMechanismType() {
		return LandUseSelectionMechanism.Type.CPTOptimizationSelectionMechanism;
	}
		
	public void setCurrentReferenceValue(Double refValue) {
		this.valueFunction.setReferenceValue(refValue);
	}
	
	public double apply(double x[]) {
		// Calculo la funcion de valor y la probabilidad objetiva asociada a cada estado del mundo.
		double v[] = this.valueFunction.apply(x);
		double w[] = this.objectiveWeightingFunction.apply(x);
		double W = 0.0;
		for (int i = 0; i < w.length; i++) {
			W += w[i];
		}
						
		// Utilizo la los valores de v[] para ordenar los estados posibles del mundo
		// asumiendo que la funcion es valor es ESTRICTAMENTE CRECIENTE. Si existe un escenario
		// cuyo valor es cero, no se considera ni ganancia ni perdida
		List<Double> gainsValueList = new ArrayList<Double>();
		List<Double> gainsWeightList = new ArrayList<Double>();
		List<Double> lossesValueList = new ArrayList<Double>();
		List<Double> lossesWeightList = new ArrayList<Double>();
		for (int i = 0; i < v.length; i++) {
			List<Double> valueList = (v[i] < 0.0) ? lossesValueList : (v[i] > 0.0) ? gainsValueList : null;
			List<Double> weightList = (v[i] < 0.0) ? lossesWeightList : (v[i] > 0.0) ? gainsWeightList : null;
			if (valueList != null) {
				int pos = 0;
				boolean foundPos = false;
				while ((pos < valueList.size()) && !foundPos) {
					Double e = valueList.get(pos);
					if (v[i] < e) {
						foundPos = true;
					} else {
						pos++;
					}
				}
				valueList.add(pos, v[i]);
				weightList.add(pos, w[i] / W);
			}		
		}
		
		// Generacion de vectores de valor y peso objetivo para ser procesados por la funcion de
		// peso subjectiva
		Double vGains[] = gainsValueList.toArray(new Double[0]);
		Double pGains[] = gainsWeightList.toArray(new Double[0]);
		
		Collections.reverse(lossesValueList);
		Collections.reverse(lossesWeightList);
		Double vLosses[] = lossesValueList.toArray(new Double[0]);
		Double pLosses[] = lossesWeightList.toArray(new Double[0]);
				
		Object X[][] = { { vGains, pGains, this.gammaGains }, { vLosses, pLosses, this.gammaLosses } };
		
		// Ahora calculo V+ y V- de acuerdo a los valores obtenidos
		double V[] = new double[2];
		for (int i = 0; i < V.length; i++) {
			Double u[] = (Double []) X[i][0];
			Double p[] = (Double []) X[i][1];
			Double gamma = (Double) X[i][2];

			if (u.length > 0) {
				CPTWeightingFunction g = new CPTWeightingFunction(gamma);
				int n = p.length - 1;
				V[i] = u[n] * g.apply(p[n]);
				for (int k = 1; k <= n; k++) {
					double arg0 = 0.0;
					for (int j = 0; j < k; j++) {
						arg0 += p[n-j];
					}
					double arg1 = arg0 + p[n-k];
					
					V[i] += u[n-k] * (g.apply(arg1) - g.apply(arg0));
				}
			}
		}
		return (V[0] + V[1]); 
	}
	
	public Double getCertaintyEquivalent(Double u) {
		return this.valueFunction.getCertaintyEquivalent(u);
	}
}