package fiuba.lamm.agromodel.math.custom;

import fiuba.lamm.agromodel.math.*;

public class CPTValueFunction extends VectorialField {
	protected Double lambda;
	protected Double alpha;
	protected Double refValue;
	
	public CPTValueFunction(Double lambda, Double alpha) {
		this.lambda = lambda;
		this.alpha = alpha;
		this.refValue = null;
	}
	
	public void setReferenceValue(Double refValue) {
		this.refValue = refValue;
	}

	public double[] apply(double x[]) {
		double y[] = new double[x.length];
		for (int i = 0; i < x.length; i++) {
			double v = x[i] - refValue;
			if (v >= 0.0) {
				y[i] = Math.pow(v, this.alpha);
			} else {
				y[i] = -this.lambda * Math.pow(-v, this.alpha);
			}
		}
		
		return y;
	}
	
	protected double getCertaintyEquivalent(double y) {
		double x;
		if (y >= 0.0) {
			x = Math.pow(y, 1.0 / this.alpha);
		} else {
			x = - Math.pow(-y / this.lambda, 1.0 / this.alpha);
		}
		
		return (x + this.refValue);
	}
}