package fiuba.lamm.agromodel.math.custom;

import fiuba.lamm.agromodel.math.*;

public class CapitalProgressRateFunction extends ScalarField {
	public double apply(double x[]) {
		double c0 = x[0];
		double c1 = x[1];
		
		if (c0 != 0) {
			/*
			 * El capital inicial es distinto de cero, luego se puede efectuar la division correspondiente para
			 * calcular el progress rate. El mismo se define como:
			 * 
			 * pr = signum(c0) * (c1 - c0) / c0
			 * 
			 */
			
			return (Math.signum(c0) * ((c1 - c0) / c0));
		} else {
			if (c1 > 0) {
				// El capital inicial es cero. El final, positivo. Luego, el progreso es mas infinito.
				return Double.POSITIVE_INFINITY;
			} else if (c1 < 0) {
				// El capital inicial es cero. El final, negativo. Luego, el progreso es menos infinito.
				return Double.NEGATIVE_INFINITY;
			} else {
				// El capital inicial y el final son iguales a 0. No hubo avances, entonces el progress rate es 0.
				return 0.0;
			}
		}
	}
}
