package fiuba.lamm.agromodel.math.custom;

import fiuba.lamm.agromodel.decision.*;
import fiuba.lamm.agromodel.math.*;

public class EUFitnessFunction extends FitnessFunction {
	protected EUValueFunction valueFunction;
	protected VectorialField weightingFunction;

	public EUFitnessFunction(EUValueFunction valueFunction, VectorialField weightingFunction) {
		this.valueFunction = valueFunction;
		this.weightingFunction = weightingFunction;		
	}
	
	public LandUseSelectionMechanism.Type getLandUseSelectionMechanismType() {
		return LandUseSelectionMechanism.Type.EUOptimizationSelectionMechanism;
	}
	
	public void setCurrentReferenceValue(Double refValue) {
		this.valueFunction.setW0(refValue);
	}
	
	public double apply(double x[]) {
		double f[] = this.valueFunction.apply(x);
		double w[] = this.weightingFunction.apply(x);
		double y = 0.0;
		
		double W = 0.0;
		for (int i = 0; i < f.length; i++) {
			y += f[i] * w[i];
			W += w[i];
		}		
		y /= W;
		
		return y; 
	}
	
	public Double getCertaintyEquivalent(Double u) {
		return this.valueFunction.getCertaintyEquivalent(u);
	}
}