package fiuba.lamm.agromodel.math.custom;

import fiuba.lamm.agromodel.execution.*;
import fiuba.lamm.agromodel.math.*;

public class EUValueFunction extends VectorialField {
	protected Double r;
	protected Double w0;
	
	public EUValueFunction(Double r) {
		this.r = r;
	}
	
	public void setW0(Double w0) {
		this.w0 = w0;
	}

	private double apply(double w) {
		if (this.r == 1.0) {
			return Math.log(w);
		} else {
			return (Math.pow(w, 1.0 - this.r) / (1.0 - this.r));
		}
	}
	
	public double[] apply(double x[]) {
		// Calculo la utilidad de cada valor corregido
		double y[] = new double[x.length];
		for (int i = 0; i < x.length; i++) {
			double w = x[i] + this.w0;
			if (w <= 0) {			
				// w <= 0. Lanzo un error fulminante
				throw new FulminantError("Error fulminante: w = " + w + "; x[i] = " + x[i] + "; w0 = " + this.w0 + "; r = "+ r);
			}
						
			y[i] = apply(w);
		}
		
		return y;
	}
	
	protected double getCertaintyEquivalent(double u) {
		double w;		
		if (this.r == 1.0) {
			w = Math.exp(u);
		} else {
			w = Math.pow(u * (1.0 - this.r), 1.0 / (1.0 - this.r));
		}
				
		double x = w - this.w0;
		return x;		
	}
}