package fiuba.lamm.agromodel.math.custom;

import fiuba.lamm.agromodel.*;
import fiuba.lamm.agromodel.math.*;
import java.util.*;

/**
 * @author santiago
 * 
 * La superficie minima alquilable (MLA por sus siglas en ingles) se calcula en funcion a un farmer y a una region.
 *
 * Si la region es una de las preferidas del farmer, entonces  la MLA es una funcion exponencial del area total operada (lo que se venia utilizando hasta ahora)
 * Sino, la MLA es un valor fijo estipulado en 1500 has (que es la unidad operacional minima)
 * TODO: En el futuro, en vez de tomar 1500 has para las regiones no preferidas, habria que tomar la minima distancia a las regiones preferidas y en base a esa distancia estipular la MLA
 * 
 *  Importante: Si el listado de regiones preferidas esta vacio, entonces la region se considera automaticamente como preferida.
 */
public class MLAScalingFunction extends Function {
	protected Function mlaFunction;
	
	public MLAScalingFunction(Region region, Set<Region> preferredRegions) {
		if (preferredRegions.contains(region) || preferredRegions.isEmpty()) {
			this.mlaFunction = new ExponentialFunction(
				new double[] { 50.0, 0.2 },
				new double[] { 420.0, 0.1 },
				0.035
			);
		} else {
			// No tomamos 1500 has porque hay pocos campos y no podemos garantizar que alquile campos mas chicos que en conjunto sumen 1500.
			// Para ser mas consistentes, bajamos el limite a 750.
			this.mlaFunction = new ConstantFunction(750.0);
		}
	}
	
	public double apply(double x) {
		return this.mlaFunction.apply(x);
	}
}