package fiuba.lamm.agromodel.networks;

import fiuba.lamm.agromodel.*;
import fiuba.lamm.agromodel.agents.*;
import java.util.*;
import repast.simphony.context.*;
import repast.simphony.context.space.graph.*;
import repast.simphony.space.graph.*;

public class MooreNetworkModel extends FarmNetworkModel {
	public Network<Agent> createNetwork(Context<Agent> context,	List<Farm> farms) {
		NetworkFactory networkFactory = NetworkFactoryFinder.createNetworkFactory(new HashMap<String, Object>());
		Network<Agent> neighbourhood = networkFactory.createNetwork("Neighborhood", context, false);
		
		/* 
		 * 1. Create a map with the following content
		 *
		 * - Each key is a Region
		 * - Each value is an unsorted list of farms located in that Region
		 */
		Map<Region, List<Farm>> farmsMap = new HashMap<Region, List<Farm>>();
		for (Farm farm: farms) {
			Region region = farm.getRegion();
			if (! farmsMap.containsKey(region)) farmsMap.put(region, new ArrayList<Farm>());
				
			List<Farm> farmsByRegion = farmsMap.get(region);
			farmsByRegion.add(farm);
		}
		
		// 2. Create one cluster per region
		FarmGeographicComparator comparator = new FarmGeographicComparator();
		Iterator<Region> regions = farmsMap.keySet().iterator();
		while (regions.hasNext()) {
			Region region = regions.next();
			List<Farm> regionFarms = farmsMap.get(region);
			
			// 2a. Sort farms list
			Farm farmsAsArray[] = regionFarms.toArray(new Farm[0]);
			Arrays.sort(farmsAsArray, comparator);
					
			// 2b. Arrange nodes into a matrix
			int N = (int) Math.ceil(Math.sqrt((double) farmsAsArray.length));
			Farm matrix[][] = new Farm[N][N];
			for (int i = 0; i < N; i++) {
				for (int j = 0; j < N; j++) {
					int index = (i * N) + j;
					Farm farm = (index < farmsAsArray.length) ? farmsAsArray[index] : null;
					matrix[i][j] = farm;
				}
			}
			
			// 2c. Create Moore's neighborhood as a network
			for (int i = 0; i < N; i++) {
				for (int j = 0; j < N; j++) {
					Farm farm = matrix[i][j];
					if (farm != null) {
						int locations[][] = {
							{ i-1, j-1 }, { i-1, j }, { i-1, j+1 },
							{ i, j-1 }, { i, j+1 },
							{ i+1, j-1 }, { i+1, j }, { i+1, j+1 }							
						};
						
						for (int k = 0; k <  locations.length; k++) {
							int y = locations[k][0];
							int x = locations[k][1];
							if ((x >= 0) && (y >= 0) && (x < N) && (y < N)) {
								Farm peer = matrix[y][x];
								if (peer != null) neighbourhood.addEdge(farm, peer);
							}
						}
					}
				}
			}
		}
		
		return neighbourhood;
	}
}