package fiuba.lamm.agromodel.tests;

import fiuba.lamm.agromodel.math.*;
import fiuba.lamm.agromodel.math.custom.*;
import org.junit.*;
import static org.junit.Assert.*;

public class CPTCalculationTest extends BaseTest {
	@Test
	public void doCPTCalculation() {
		// CPT parameters
		double lambda = 2.25;
		double alpha = 0.88;
		double gammaGains = 0.61;
		double gammaLosses = 0.69;
		
		// Inputs
		double refValues[] = { 100.0, 0.0 };
		double outcomes[][] = { 
			{ -100.0, 500.0, 100.0, 0.0, 300.0, 400.0, 200.0, -200.0, 600.0, -300.0 },
			{ -37.56, 26.66, -121.89, 44.83, 76.09, -9.43, -75.64, 15.01, -13.56, 88.34 },
		};
		
		// Expected outputs
		double ev[] = { -53.498, -25.2233 };
		double ce[] = { 63.3721, -15.5865 };
								
		CPTValueFunction valueFunction = new CPTValueFunction(lambda, alpha);		
		CPTFitnessFunction fitnessFunction = new CPTFitnessFunction(valueFunction, new ConstantVectorialField(1.0), gammaGains, gammaLosses);
		for (int i = 0; i < outcomes.length; i++) {
			fitnessFunction.setCurrentReferenceValue(refValues[i]);
			double cptValue = fitnessFunction.apply(outcomes[i]);
			double cptCertaintyEquivalent = fitnessFunction.getCertaintyEquivalent(cptValue);
			
			assertEquals("CPT expected value mismatch", ev[i], cptValue, 0.5E-4);
			assertEquals("CPT certainty equivalent mismatch", ce[i], cptCertaintyEquivalent, 0.5E-4);
		}
	}
}