package fiuba.lamm.agromodel.tests;

import fiuba.lamm.agromodel.math.custom.*;
import org.junit.*;
import static org.junit.Assert.*;

public class ProgressRateCalculationTest extends BaseTest {
	protected CapitalProgressRateFunction capitalProgressRateFunction;
	
	@Before
	public void setUp() throws Exception {
		this.capitalProgressRateFunction = new CapitalProgressRateFunction();
	}

	@After
	public void tearDown() throws Exception {
		this.capitalProgressRateFunction = null;
	}

	@Test
	public void testZeroInitialCapital() {
		// Chequeo progreso nulo
		assertEquals(0.0, this.capitalProgressRateFunction.apply(new double[] { 0.0, 0.0 }), 0.0);
		
		// Chequeo de progreso infinito
		Double negativeInfinity = this.capitalProgressRateFunction.apply(new double[] { 0.0, -1.0 });
		Double positiveInfinity = this.capitalProgressRateFunction.apply(new double[] { 0.0, 1.0 });
		assertEquals(Double.NEGATIVE_INFINITY, negativeInfinity, 0.0);
		assertEquals(Double.POSITIVE_INFINITY, positiveInfinity, 0.0);
		for (double x = -10.0; x < 10.0; x += 1.0) {
			assertTrue(x > negativeInfinity);
			assertTrue(x < positiveInfinity);
		}
	}
	
	@Test
	public void testNonZeroInitialCapital() {
		// Casos donde el capital inicial es negativo
		assertEquals(4.5, this.capitalProgressRateFunction.apply(new double[] { -100.0, 350.0 }), 0.0);
		assertEquals(0.4, this.capitalProgressRateFunction.apply(new double[] { -100.0, -60.0 }), 0.0);
		assertEquals(-1.0, this.capitalProgressRateFunction.apply(new double[] { -100.0, -200.0 }), 0.0);
		assertEquals(0.2, this.capitalProgressRateFunction.apply(new double[] { -100.0, -80.0 }), 0.0);
		assertEquals(0.0, this.capitalProgressRateFunction.apply(new double[] { -100.0, -100.0 }), 0.0);
		
		// Casos donde el capital inicial es positivo
		assertEquals(2.5, this.capitalProgressRateFunction.apply(new double[] { 100.0, 350.0 }), 0.0);
		assertEquals(0.4, this.capitalProgressRateFunction.apply(new double[] { 100.0, 140.0 }), 0.0);
		assertEquals(-3.0, this.capitalProgressRateFunction.apply(new double[] { 100.0, -200.0 }), 0.0);
		assertEquals(0.2, this.capitalProgressRateFunction.apply(new double[] { 100.0, 120.0 }), 0.0);
		assertEquals(0.0, this.capitalProgressRateFunction.apply(new double[] { 100.0, 100.0 }), 0.0);
	}
}
