package fiuba.lamm.agromodel.utils;

import java.util.*;

import fiuba.lamm.agromodel.agents.Manager;
import fiuba.lamm.agromodel.execution.*;

public class Randomizer {
	protected Random random; 
	
	public Randomizer(Integer randomSeed) {
		this.random = new Random((long) randomSeed);
	}
	
	public Double getRandomDouble() {
		return this.random.nextDouble();
	}
	
	public Integer getRandomInt(int maxValue) {
		return this.random.nextInt(maxValue);		
	}
	
	public <T extends Comparable<? super T>> T getRandomEvent(Map<T, Double> probabilities) {		
		List<T> events = new ArrayList<T>(probabilities.keySet());
		Collections.sort(events); // The events MUST be sorted in a consistent order
		T event = null;
		Double p = getRandomDouble();
		Double min = 0.0;
		
		int i = 0;
		while ((i < events.size()) && (event == null)) {
			T candidate = events.get(i);
			Double pc  = probabilities.get(candidate);
			if ((pc == null) || pc.isInfinite() || pc.isNaN()) {
				throw new FulminantError("La probabilidad no es un numero. probabilidades = " + probabilities);
			}
			
			Double max = min + pc;
			if ((p >= min) && (p < max)) {
				event = candidate;
			} else {
				min = max;
			}
			i++;
		}
		
		// For avoiding rounding issues, if no event was chosen, pick last event
		if (event == null) {
			Manager manager = Manager.getInstance();
			throw new FulminantError("Cropping cycle =" + manager.getCurrentCroppingCycle() + ". Scenario = " + manager.getScenario() + ". Problema de redondeo. Eventos: " + probabilities + ". p = " + p);
			//event = events.get(events.size() - 1);
		}
		
		return event;
	}	
}