package fiuba.lamm.agromodel.utils;

import fiuba.lamm.agromodel.execution.*;
import java.io.*;
import java.util.*;
import repast.simphony.engine.environment.*;

public final class System {
	public static void init() {
		// Run garbage collector
		Runtime runtime = Runtime.getRuntime();
		runtime.gc();
		
		// Read config file
		Properties config = new Properties();
		File configFile = null;
		String projectRootLocation = java.lang.System.getenv("PROJECT_ROOT");
		if ((projectRootLocation != null) && !projectRootLocation.trim().equals("")) {
			File projectRoot = new File(projectRootLocation);
			configFile = new File(projectRoot, "config/init.properties");
		} else {
			configFile = new File("config/init.properties");
			try {
				projectRootLocation = configFile.getCanonicalFile().getParentFile().getParent();			
			} catch (IOException ioe) {
				throw new FulminantError("Error trying to find the working directory: " + ioe);
			}
		}
				
		try {
			FileInputStream fis = new FileInputStream(configFile);
			config.load(fis);
			
			Enumeration<?> keys = config.propertyNames();
			while (keys.hasMoreElements()) {
				String key = (String) keys.nextElement();
				java.lang.System.setProperty(key, config.getProperty(key));
			}
		} catch (IOException ioe) {
			throw new FulminantError("Error al leer el archivo de configuracion: " + ioe.getMessage());			
		}
		
		// Agregado de path del PROJECT_ROOT
		java.lang.System.setProperty("fiuba.lamm.agromodel.workdir", projectRootLocation);
	}
	
	public static void finish() {		
		// Clear cached data
		Scenario scenario = (Scenario) RunState.getInstance().getFromRegistry("scenario");
		fiuba.lamm.agromodel.agents.factories.AdvisorFactory.clearScenarioCache(scenario);
		fiuba.lamm.agromodel.agents.factories.FarmerFactory.clearScenarioCache(scenario);
		fiuba.lamm.agromodel.agents.factories.FarmFactory.clearScenarioCache(scenario);
		fiuba.lamm.agromodel.decision.factories.ContextTypeFactory.clearScenarioCache(scenario);
		fiuba.lamm.agromodel.factories.CreaGroupFactory.clearScenarioCache(scenario);
		fiuba.lamm.agromodel.factories.CreaRegionFactory.clearScenarioCache(scenario);
		fiuba.lamm.agromodel.factories.CroppingCycleFactory.clearScenarioCache(scenario);
		fiuba.lamm.agromodel.factories.SoilTypeFactory.clearScenarioCache(scenario);		
		scenario.setManager(null);
		
		// End simulation
		RunEnvironment runEnvironment = RunEnvironment.getInstance();
		RunState.getInstance().removeFromRegistry("scenario");
		runEnvironment.endRun();
		
		// Run pending finalization methods
		java.lang.System.runFinalization();				
	}
	
	public static String elapsedTime(Double elapsedSeconds) {		
		if (elapsedSeconds < 1.0) {
			return (elapsedSeconds + "s");
		} else {
			Integer seconds = (int) Math.round(elapsedSeconds);
			if (seconds < 60) {
				return (seconds + "s");
			} else if (seconds < 3600) {
				Integer m = seconds / 60;
				Integer s = seconds % 60;
				return (m + "m " + s + "s");
			} else {
				Integer h = seconds / 3600;
				Integer m = (seconds / 3600) / 60;
				Integer s = seconds % 60;
				return (h + "h " + m + "m " + s + "s");
			}
		}
	}
}